@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -71,7 +71,7 @@ namespace gazebo
       rendering::ScenePtr scene = rendering::get_scene();
 
       // Wait until the scene is initialized.
-      if (!scene || !scene->GetInitialized())
+      if (!scene || !scene->Initialized())
         return;
 
       // Look for a specific visual by name.
