@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@
 #include "gazebo/transport/TransportIface.hh"
 #include "gazebo/gui/GuiIface.hh"
 #include "gazebo/gui/JointControlWidget.hh"
+#include "gazebo/gui/JointControlWidgetPrivate.hh"
 
 using namespace gazebo;
 using namespace gui;
@@ -32,17 +33,19 @@ using namespace gui;
 /////////////////////////////////////////////////
 JointForceControl::JointForceControl(const std::string &_name,
     QGridLayout *_layout, QWidget *_parent, int _index)
-  : QWidget(_parent), name(_name)
+  : QWidget(_parent),
+    dataPtr(new JointForceControlPrivate())
 {
-  this->forceSpin = new QDoubleSpinBox;
-  this->forceSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->forceSpin->setSingleStep(0.001);
-  this->forceSpin->setDecimals(3);
-  this->forceSpin->setValue(0.000);
+  this->dataPtr->name = _name;
+  this->dataPtr->forceSpin = new QDoubleSpinBox;
+  this->dataPtr->forceSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->forceSpin->setSingleStep(0.001);
+  this->dataPtr->forceSpin->setDecimals(3);
+  this->dataPtr->forceSpin->setValue(0.000);
 
-  _layout->addWidget(forceSpin, _index, 2);
+  _layout->addWidget(this->dataPtr->forceSpin, _index, 2);
 
-  connect(this->forceSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->forceSpin, SIGNAL(valueChanged(double)),
           this, SLOT(OnChanged(double)));
 }
 
@@ -55,208 +58,223 @@ JointForceControl::~JointForceControl()
 /////////////////////////////////////////////////
 void JointForceControl::Reset()
 {
-  this->forceSpin->setValue(0.0);
+  this->dataPtr->forceSpin->setValue(0.0);
 }
 
 /////////////////////////////////////////////////
 void JointForceControl::OnChanged(double _value)
 {
-  emit changed(_value, this->name);
+  emit changed(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 JointPIDPosControl::JointPIDPosControl(const std::string &_name,
     QGridLayout *_layout, QWidget *_parent, int _index)
-  : QWidget(_parent), name(_name)
+  : QWidget(_parent),
+    dataPtr(new JointPIDPosControlPrivate())
 {
-  this->posSpin = new QDoubleSpinBox;
-  this->posSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->posSpin->setSingleStep(0.001);
-  this->posSpin->setDecimals(3);
-  this->posSpin->setValue(0.000);
+  this->dataPtr->name = _name;
+  this->dataPtr->posSpin = new QDoubleSpinBox;
+  this->dataPtr->posSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->posSpin->setSingleStep(0.001);
+  this->dataPtr->posSpin->setDecimals(3);
+  this->dataPtr->posSpin->setValue(0.000);
 
-  this->pGainSpin = new QDoubleSpinBox;
-  this->pGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->pGainSpin->setSingleStep(0.01);
-  this->pGainSpin->setDecimals(3);
-  this->pGainSpin->setValue(1.000);
+  this->dataPtr->pGainSpin = new QDoubleSpinBox;
+  this->dataPtr->pGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->pGainSpin->setSingleStep(0.01);
+  this->dataPtr->pGainSpin->setDecimals(3);
+  this->dataPtr->pGainSpin->setValue(1.000);
 
-  this->iGainSpin = new QDoubleSpinBox;
-  this->iGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->iGainSpin->setSingleStep(0.01);
-  this->iGainSpin->setDecimals(3);
-  this->iGainSpin->setValue(0.100);
+  this->dataPtr->iGainSpin = new QDoubleSpinBox;
+  this->dataPtr->iGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->iGainSpin->setSingleStep(0.01);
+  this->dataPtr->iGainSpin->setDecimals(3);
+  this->dataPtr->iGainSpin->setValue(0.100);
 
-  this->dGainSpin = new QDoubleSpinBox;
-  this->dGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->dGainSpin->setSingleStep(0.01);
-  this->dGainSpin->setDecimals(3);
-  this->dGainSpin->setValue(0.010);
+  this->dataPtr->dGainSpin = new QDoubleSpinBox;
+  this->dataPtr->dGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->dGainSpin->setSingleStep(0.01);
+  this->dataPtr->dGainSpin->setDecimals(3);
+  this->dataPtr->dGainSpin->setValue(0.010);
 
-  _layout->addWidget(this->posSpin, _index, 2);
-  _layout->addWidget(pGainSpin, _index, 3);
-  _layout->addWidget(iGainSpin, _index, 4);
-  _layout->addWidget(dGainSpin, _index, 5);
+  _layout->addWidget(this->dataPtr->posSpin, _index, 2);
+  _layout->addWidget(this->dataPtr->pGainSpin, _index, 3);
+  _layout->addWidget(this->dataPtr->iGainSpin, _index, 4);
+  _layout->addWidget(this->dataPtr->dGainSpin, _index, 5);
 
-  connect(this->posSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->posSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnChanged(double)));
-  connect(pGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->pGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnPChanged(double)));
-  connect(iGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->iGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnIChanged(double)));
-  connect(dGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->dGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnDChanged(double)));
 
-  this->radians = true;
+  this->dataPtr->radians = true;
+}
+
+/////////////////////////////////////////////////
+JointPIDPosControl::~JointPIDPosControl()
+{
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::Reset()
 {
-  this->posSpin->setValue(0.0);
-  this->pGainSpin->setValue(1.000);
-  this->iGainSpin->setValue(0.100);
-  this->dGainSpin->setValue(0.010);
+  this->dataPtr->posSpin->setValue(0.0);
+  this->dataPtr->pGainSpin->setValue(1.000);
+  this->dataPtr->iGainSpin->setValue(0.100);
+  this->dataPtr->dGainSpin->setValue(0.010);
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::SetToDegrees()
 {
-  if (this->radians)
+  if (this->dataPtr->radians)
   {
-    this->radians = false;
-    this->posSpin->setValue(GZ_RTOD(this->posSpin->value()));
+    this->dataPtr->radians = false;
+    this->dataPtr->posSpin->setValue(GZ_RTOD(this->dataPtr->posSpin->value()));
   }
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::SetToRadians()
 {
-  if (!this->radians)
+  if (!this->dataPtr->radians)
   {
-    this->radians = true;
-    this->posSpin->setValue(GZ_DTOR(this->posSpin->value()));
+    this->dataPtr->radians = true;
+    this->dataPtr->posSpin->setValue(GZ_DTOR(this->dataPtr->posSpin->value()));
   }
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::OnChanged(double _value)
 {
-  if (this->radians)
-    emit changed(_value, this->name);
+  if (this->dataPtr->radians)
+    emit changed(_value, this->dataPtr->name);
   else
-    emit changed(GZ_DTOR(_value), this->name);
+    emit changed(GZ_DTOR(_value), this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::OnPChanged(double _value)
 {
-  emit pChanged(_value, this->name);
+  emit pChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::OnIChanged(double _value)
 {
-  emit iChanged(_value, this->name);
+  emit iChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDPosControl::OnDChanged(double _value)
 {
-  emit dChanged(_value, this->name);
+  emit dChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 JointPIDVelControl::JointPIDVelControl(const std::string &_name,
     QGridLayout *_layout, QWidget *_parent, int _index)
-  : QWidget(_parent), name(_name)
+  : QWidget(_parent),
+    dataPtr(new JointPIDVelControlPrivate())
 {
-  this->posSpin = new QDoubleSpinBox;
-  this->posSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->posSpin->setSingleStep(0.001);
-  this->posSpin->setDecimals(3);
-  this->posSpin->setValue(0.000);
+  this->dataPtr->name = _name;
+  this->dataPtr->posSpin = new QDoubleSpinBox;
+  this->dataPtr->posSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->posSpin->setSingleStep(0.001);
+  this->dataPtr->posSpin->setDecimals(3);
+  this->dataPtr->posSpin->setValue(0.000);
 
-  this->pGainSpin = new QDoubleSpinBox;
-  this->pGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->pGainSpin->setSingleStep(0.01);
-  this->pGainSpin->setDecimals(3);
-  this->pGainSpin->setValue(1.000);
+  this->dataPtr->pGainSpin = new QDoubleSpinBox;
+  this->dataPtr->pGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->pGainSpin->setSingleStep(0.01);
+  this->dataPtr->pGainSpin->setDecimals(3);
+  this->dataPtr->pGainSpin->setValue(1.000);
 
-  this->iGainSpin = new QDoubleSpinBox;
-  this->iGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->iGainSpin->setSingleStep(0.01);
-  this->iGainSpin->setDecimals(3);
-  this->iGainSpin->setValue(0.100);
+  this->dataPtr->iGainSpin = new QDoubleSpinBox;
+  this->dataPtr->iGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->iGainSpin->setSingleStep(0.01);
+  this->dataPtr->iGainSpin->setDecimals(3);
+  this->dataPtr->iGainSpin->setValue(0.100);
 
-  this->dGainSpin = new QDoubleSpinBox;
-  this->dGainSpin->setRange(-GZ_DBL_MAX, GZ_DBL_MAX);
-  this->dGainSpin->setSingleStep(0.01);
-  this->dGainSpin->setDecimals(3);
-  this->dGainSpin->setValue(0.010);
+  this->dataPtr->dGainSpin = new QDoubleSpinBox;
+  this->dataPtr->dGainSpin->setRange(-IGN_DBL_MAX, IGN_DBL_MAX);
+  this->dataPtr->dGainSpin->setSingleStep(0.01);
+  this->dataPtr->dGainSpin->setDecimals(3);
+  this->dataPtr->dGainSpin->setValue(0.010);
 
-  _layout->addWidget(posSpin, _index, 2);
-  _layout->addWidget(pGainSpin, _index, 3);
-  _layout->addWidget(iGainSpin, _index, 4);
-  _layout->addWidget(dGainSpin, _index, 5);
+  _layout->addWidget(this->dataPtr->posSpin, _index, 2);
+  _layout->addWidget(this->dataPtr->pGainSpin, _index, 3);
+  _layout->addWidget(this->dataPtr->iGainSpin, _index, 4);
+  _layout->addWidget(this->dataPtr->dGainSpin, _index, 5);
 
-  connect(posSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->posSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnChanged(double)));
-  connect(pGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->pGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnPChanged(double)));
-  connect(iGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->iGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnIChanged(double)));
-  connect(dGainSpin, SIGNAL(valueChanged(double)),
+  connect(this->dataPtr->dGainSpin, SIGNAL(valueChanged(double)),
         this, SLOT(OnDChanged(double)));
 }
 
+/////////////////////////////////////////////////
+JointPIDVelControl::~JointPIDVelControl()
+{
+}
+
 /////////////////////////////////////////////////
 void JointPIDVelControl::Reset()
 {
-  this->posSpin->setValue(0.0);
-  this->pGainSpin->setValue(1.000);
-  this->iGainSpin->setValue(0.100);
-  this->dGainSpin->setValue(0.010);
+  this->dataPtr->posSpin->setValue(0.0);
+  this->dataPtr->pGainSpin->setValue(1.000);
+  this->dataPtr->iGainSpin->setValue(0.100);
+  this->dataPtr->dGainSpin->setValue(0.010);
 }
 
 /////////////////////////////////////////////////
 void JointPIDVelControl::OnChanged(double _value)
 {
-  emit changed(_value, this->name);
+  emit changed(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDVelControl::OnPChanged(double _value)
 {
-  emit pChanged(_value, this->name);
+  emit pChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDVelControl::OnIChanged(double _value)
 {
-  emit iChanged(_value, this->name);
+  emit iChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointPIDVelControl::OnDChanged(double _value)
 {
-  emit dChanged(_value, this->name);
+  emit dChanged(_value, this->dataPtr->name);
 }
 
 /////////////////////////////////////////////////
 void JointControlWidget::SetModelName(const std::string &_modelName)
 {
-  if (this->jointPub)
-    this->jointPub.reset();
+  if (this->dataPtr->jointPub)
+    this->dataPtr->jointPub.reset();
 
   msgs::Model modelMsg;
 
-  this->modelLabel->setText(QString::fromStdString(std::string("Model: ")));
+  this->dataPtr->modelLabel->setText(
+      QString::fromStdString(std::string("Model: ")));
 
   // Only request info if the model has a name.
   if (!_modelName.empty())
   {
-    this->jointPub = this->node->Advertise<msgs::JointCmd>(
+    this->dataPtr->jointPub = this->dataPtr->node->Advertise<msgs::JointCmd>(
         std::string("~/") + _modelName + "/joint_cmd");
 
     boost::shared_ptr<msgs::Response> response = transport::request(
@@ -269,8 +287,8 @@ void JointControlWidget::SetModelName(const std::string &_modelName)
     }
   }
 
-  this->modelLabel->setText(QString::fromStdString(
-        std::string("Model: ") + modelMsg.name()));
+  this->dataPtr->modelLabel->setText(QString::fromStdString(
+      std::string("Model: ") + modelMsg.name()));
 
   this->LayoutForceTab(modelMsg);
 
@@ -281,36 +299,40 @@ void JointControlWidget::SetModelName(const std::string &_modelName)
 
 /////////////////////////////////////////////////
 JointControlWidget::JointControlWidget(QWidget *_parent)
-  : QWidget(_parent)
+  : QWidget(_parent),
+    dataPtr(new JointControlWidgetPrivate())
 {
   this->setObjectName("jointControl");
 
   this->setWindowTitle("Joint Control");
-  this->node = transport::NodePtr(new transport::Node());
-  this->node->Init();
+  this->dataPtr->node = transport::NodePtr(new transport::Node());
+  this->dataPtr->node->Init();
 
-  this->tabWidget = new QTabWidget;
-  this->tabWidget->setObjectName("embeddedTab");
+  this->dataPtr->tabWidget = new QTabWidget;
+  this->dataPtr->tabWidget->setObjectName("embeddedTab");
 
   // Create the Force control scroll area
-  this->forceGridLayout = new QGridLayout;
-  this->AddScrollTab(this->tabWidget, this->forceGridLayout, tr("Force"));
+  this->dataPtr->forceGridLayout = new QGridLayout;
+  this->AddScrollTab(
+      this->dataPtr->tabWidget, this->dataPtr->forceGridLayout, tr("Force"));
 
   // Create a PID Pos scroll area
-  this->positionGridLayout = new QGridLayout;
-  this->AddScrollTab(this->tabWidget, this->positionGridLayout, tr("Position"));
+  this->dataPtr->positionGridLayout = new QGridLayout;
+  this->AddScrollTab(this->dataPtr->tabWidget,
+      this->dataPtr->positionGridLayout, tr("Position"));
 
   // Create a PID Vel scroll area
-  this->velocityGridLayout = new QGridLayout;
-  this->AddScrollTab(this->tabWidget, this->velocityGridLayout, tr("Velocity"));
+  this->dataPtr->velocityGridLayout = new QGridLayout;
+  this->AddScrollTab(this->dataPtr->tabWidget,
+      this->dataPtr->velocityGridLayout, tr("Velocity"));
 
   // Add the the force and pid scroll areas to the tab
   QVBoxLayout *mainLayout = new QVBoxLayout;
 
   QHBoxLayout *hboxLayout = new QHBoxLayout;
 
-  this->modelLabel = new QLabel(tr("Model: "));
-  hboxLayout->addWidget(this->modelLabel);
+  this->dataPtr->modelLabel = new QLabel(tr("Model: "));
+  hboxLayout->addWidget(this->dataPtr->modelLabel);
 
   hboxLayout->addItem(new QSpacerItem(10, 20, QSizePolicy::Expanding,
                       QSizePolicy::Minimum));
@@ -320,7 +342,7 @@ JointControlWidget::JointControlWidget(QWidget *_parent)
   hboxLayout->addWidget(resetButton);
 
   mainLayout->addLayout(hboxLayout);
-  mainLayout->addWidget(this->tabWidget);
+  mainLayout->addWidget(this->dataPtr->tabWidget);
   mainLayout->setContentsMargins(4, 4, 4, 4);
 
   this->setLayout(mainLayout);
@@ -335,23 +357,26 @@ JointControlWidget::~JointControlWidget()
 void JointControlWidget::OnReset()
 {
   for (std::map<std::string, JointForceControl*>::iterator iter =
-       this->sliders.begin(); iter != this->sliders.end(); ++iter)
+       this->dataPtr->sliders.begin();
+       iter != this->dataPtr->sliders.end(); ++iter)
   {
     msgs::JointCmd msg;
     msg.set_name(iter->first);
     msg.set_reset(true);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
     iter->second->Reset();
   }
 
   for (std::map<std::string, JointPIDPosControl*>::iterator iter =
-       this->pidPosSliders.begin(); iter != this->pidPosSliders.end(); ++iter)
+       this->dataPtr->pidPosSliders.begin();
+       iter != this->dataPtr->pidPosSliders.end(); ++iter)
   {
     iter->second->Reset();
   }
 
   for (std::map<std::string, JointPIDVelControl*>::iterator iter =
-       this->pidVelSliders.begin(); iter != this->pidVelSliders.end(); ++iter)
+       this->dataPtr->pidVelSliders.begin();
+       iter != this->dataPtr->pidVelSliders.end(); ++iter)
   {
     iter->second->Reset();
   }
@@ -361,13 +386,13 @@ void JointControlWidget::OnReset()
 void JointControlWidget::OnForceChanged(double _value, const std::string &_name)
 {
   std::map<std::string, JointForceControl*>::iterator iter;
-  iter = this->sliders.find(_name);
-  if (iter != this->sliders.end())
+  iter = this->dataPtr->sliders.find(_name);
+  if (iter != this->dataPtr->sliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.set_force(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -376,13 +401,13 @@ void JointControlWidget::OnPIDPosChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDPosControl*>::iterator iter;
-  iter = this->pidPosSliders.find(_name);
-  if (iter != this->pidPosSliders.end())
+  iter = this->dataPtr->pidPosSliders.find(_name);
+  if (iter != this->dataPtr->pidPosSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_position()->set_target(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -391,13 +416,13 @@ void JointControlWidget::OnPPosGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDPosControl*>::iterator iter;
-  iter = this->pidPosSliders.find(_name);
-  if (iter != this->pidPosSliders.end())
+  iter = this->dataPtr->pidPosSliders.find(_name);
+  if (iter != this->dataPtr->pidPosSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_position()->set_p_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -406,13 +431,13 @@ void JointControlWidget::OnDPosGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDPosControl*>::iterator iter;
-  iter = this->pidPosSliders.find(_name);
-  if (iter != this->pidPosSliders.end())
+  iter = this->dataPtr->pidPosSliders.find(_name);
+  if (iter != this->dataPtr->pidPosSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_position()->set_d_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -421,13 +446,13 @@ void JointControlWidget::OnIPosGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDPosControl*>::iterator iter;
-  iter = this->pidPosSliders.find(_name);
-  if (iter != this->pidPosSliders.end())
+  iter = this->dataPtr->pidPosSliders.find(_name);
+  if (iter != this->dataPtr->pidPosSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_position()->set_i_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -435,8 +460,8 @@ void JointControlWidget::OnIPosGainChanged(double _value,
 void JointControlWidget::OnPIDPosUnitsChanged(int _index)
 {
   std::map<std::string, JointPIDPosControl*>::iterator iter;
-  for (iter = this->pidPosSliders.begin();
-       iter != this->pidPosSliders.end(); ++iter)
+  for (iter = this->dataPtr->pidPosSliders.begin();
+       iter != this->dataPtr->pidPosSliders.end(); ++iter)
   {
     if (_index == 0)
       iter->second->SetToRadians();
@@ -450,13 +475,13 @@ void JointControlWidget::OnPIDVelChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDVelControl*>::iterator iter;
-  iter = this->pidVelSliders.find(_name);
-  if (iter != this->pidVelSliders.end())
+  iter = this->dataPtr->pidVelSliders.find(_name);
+  if (iter != this->dataPtr->pidVelSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_velocity()->set_target(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -465,13 +490,13 @@ void JointControlWidget::OnPVelGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDVelControl*>::iterator iter;
-  iter = this->pidVelSliders.find(_name);
-  if (iter != this->pidVelSliders.end())
+  iter = this->dataPtr->pidVelSliders.find(_name);
+  if (iter != this->dataPtr->pidVelSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_velocity()->set_p_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -480,13 +505,13 @@ void JointControlWidget::OnDVelGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDVelControl*>::iterator iter;
-  iter = this->pidVelSliders.find(_name);
-  if (iter != this->pidVelSliders.end())
+  iter = this->dataPtr->pidVelSliders.find(_name);
+  if (iter != this->dataPtr->pidVelSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_velocity()->set_d_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -495,13 +520,13 @@ void JointControlWidget::OnIVelGainChanged(double _value,
     const std::string &_name)
 {
   std::map<std::string, JointPIDVelControl*>::iterator iter;
-  iter = this->pidVelSliders.find(_name);
-  if (iter != this->pidVelSliders.end())
+  iter = this->dataPtr->pidVelSliders.find(_name);
+  if (iter != this->dataPtr->pidVelSliders.end())
   {
     msgs::JointCmd msg;
     msg.set_name(_name);
     msg.mutable_velocity()->set_i_gain(_value);
-    this->jointPub->Publish(msg);
+    this->dataPtr->jointPub->Publish(msg);
   }
 }
 
@@ -536,21 +561,22 @@ void JointControlWidget::LayoutForceTab(msgs::Model &_modelMsg)
 {
   // Remove the old widgets;
   QLayoutItem *wItem;
-  while ((wItem = this->forceGridLayout->takeAt(0)) != NULL)
+  while ((wItem = this->dataPtr->forceGridLayout->takeAt(0)) != NULL)
   {
     if (wItem->widget())
       delete wItem->widget();
     delete wItem;
   }
-  this->sliders.clear();
+  this->dataPtr->sliders.clear();
 
   // Don't add any widget if there are no joints
   if (_modelMsg.joint_size() == 0)
     return;
 
-  this->forceGridLayout->addItem(new QSpacerItem(10, 20, QSizePolicy::Expanding,
-                                      QSizePolicy::Minimum), 0, 0, 2);
-  this->forceGridLayout->addWidget(new QLabel("Newton-meter", this), 0, 2);
+  this->dataPtr->forceGridLayout->addItem(new QSpacerItem(
+      10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum), 0, 0, 2);
+  this->dataPtr->forceGridLayout->addWidget(new QLabel(
+      "Newton-meter", this), 0, 2);
 
   int i = 0;
   for (; i < _modelMsg.joint_size(); ++i)
@@ -562,22 +588,23 @@ void JointControlWidget::LayoutForceTab(msgs::Model &_modelMsg)
     std::string shortName = jointName.substr(modelNameIndex,
                             jointName.size() - modelNameIndex);
 
-    this->forceGridLayout->addWidget(new QLabel(tr(shortName.c_str())), i+1, 0);
-    this->forceGridLayout->addItem(new QSpacerItem(10, 20,
-          QSizePolicy::Expanding, QSizePolicy::Minimum), i+1, 1);
+    this->dataPtr->forceGridLayout->addWidget(new QLabel(
+        tr(shortName.c_str())), i+1, 0);
+    this->dataPtr->forceGridLayout->addItem(new QSpacerItem(
+        10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum), i+1, 1);
 
     JointForceControl *slider = new JointForceControl(jointName,
-        this->forceGridLayout, this, i+1);
+        this->dataPtr->forceGridLayout, this, i+1);
 
-    this->sliders[jointName] = slider;
+    this->dataPtr->sliders[jointName] = slider;
     connect(slider, SIGNAL(changed(double, const std::string &)),
             this, SLOT(OnForceChanged(double, const std::string &)));
   }
 
   // Add a space at the bottom of the grid layout to consume extra space.
-  this->forceGridLayout->addItem(new QSpacerItem(10, 20, QSizePolicy::Expanding,
-                                      QSizePolicy::Expanding), i+1, 0);
-  this->forceGridLayout->setRowStretch(i+1, 2);
+  this->dataPtr->forceGridLayout->addItem(new QSpacerItem(
+      10, 20, QSizePolicy::Expanding, QSizePolicy::Expanding), i+1, 0);
+  this->dataPtr->forceGridLayout->setRowStretch(i+1, 2);
 }
 
 /////////////////////////////////////////////////
@@ -585,19 +612,19 @@ void JointControlWidget::LayoutPositionTab(msgs::Model &_modelMsg)
 {
   // Remove the old widgets;
   QLayoutItem *wItem;
-  while ((wItem = this->positionGridLayout->takeAt(0)) != NULL)
+  while ((wItem = this->dataPtr->positionGridLayout->takeAt(0)) != NULL)
   {
     if (wItem->widget())
       delete wItem->widget();
     delete wItem;
   }
-  this->pidPosSliders.clear();
+  this->dataPtr->pidPosSliders.clear();
 
   // Don't add any widget if there are no joints
   if (_modelMsg.joint_size() == 0)
     return;
 
-  this->positionGridLayout->addItem(new QSpacerItem(10, 20,
+  this->dataPtr->positionGridLayout->addItem(new QSpacerItem(10, 20,
         QSizePolicy::Expanding, QSizePolicy::Minimum), 0, 0, 2);
 
   QComboBox *unitsCombo = new QComboBox;
@@ -606,10 +633,13 @@ void JointControlWidget::LayoutPositionTab(msgs::Model &_modelMsg)
   connect(unitsCombo, SIGNAL(currentIndexChanged(int)),
       this, SLOT(OnPIDPosUnitsChanged(int)));
 
-  this->positionGridLayout->addWidget(unitsCombo, 0, 2);
-  this->positionGridLayout->addWidget(new QLabel("P Gain", this), 0, 3);
-  this->positionGridLayout->addWidget(new QLabel("I Gain", this), 0, 4);
-  this->positionGridLayout->addWidget(new QLabel("D Gain", this), 0, 5);
+  this->dataPtr->positionGridLayout->addWidget(unitsCombo, 0, 2);
+  this->dataPtr->positionGridLayout->addWidget(new QLabel(
+      "P Gain", this), 0, 3);
+  this->dataPtr->positionGridLayout->addWidget(new QLabel(
+      "I Gain", this), 0, 4);
+  this->dataPtr->positionGridLayout->addWidget(new QLabel(
+      "D Gain", this), 0, 5);
 
   int i = 0;
   for (; i < _modelMsg.joint_size(); ++i)
@@ -621,16 +651,16 @@ void JointControlWidget::LayoutPositionTab(msgs::Model &_modelMsg)
     std::string shortName = jointName.substr(modelNameIndex,
                             jointName.size() - modelNameIndex);
 
-    this->positionGridLayout->addWidget(
+    this->dataPtr->positionGridLayout->addWidget(
         new QLabel(tr(shortName.c_str())), i+1, 0);
-    this->positionGridLayout->addItem(
+    this->dataPtr->positionGridLayout->addItem(
         new QSpacerItem(10, 20, QSizePolicy::Expanding,
                         QSizePolicy::Minimum), i+1, 1);
 
     JointPIDPosControl *slider = new JointPIDPosControl(jointName,
-        this->positionGridLayout, this, i+1);
+        this->dataPtr->positionGridLayout, this, i+1);
 
-    this->pidPosSliders[jointName] = slider;
+    this->dataPtr->pidPosSliders[jointName] = slider;
     connect(slider, SIGNAL(changed(double, const std::string &)),
             this, SLOT(OnPIDPosChanged(double, const std::string &)));
     connect(slider, SIGNAL(pChanged(double, const std::string &)),
@@ -642,10 +672,10 @@ void JointControlWidget::LayoutPositionTab(msgs::Model &_modelMsg)
   }
 
   // Add a space at the bottom of the grid layout to consume extra space.
-  this->positionGridLayout->addItem(
+  this->dataPtr->positionGridLayout->addItem(
       new QSpacerItem(10, 20, QSizePolicy::Expanding,
                       QSizePolicy::Expanding), i+1, 0);
-  this->positionGridLayout->setRowStretch(i+1, 2);
+  this->dataPtr->positionGridLayout->setRowStretch(i+1, 2);
 }
 
 /////////////////////////////////////////////////
@@ -653,19 +683,19 @@ void JointControlWidget::LayoutVelocityTab(msgs::Model &_modelMsg)
 {
   // Remove the old widgets;
   QLayoutItem *wItem;
-  while ((wItem = this->velocityGridLayout->takeAt(0)) != NULL)
+  while ((wItem = this->dataPtr->velocityGridLayout->takeAt(0)) != NULL)
   {
     if (wItem->widget())
       delete wItem->widget();
     delete wItem;
   }
-  this->pidVelSliders.clear();
+  this->dataPtr->pidVelSliders.clear();
 
   // Don't add any widget if there are no joints
   if (_modelMsg.joint_size() == 0)
     return;
 
-  this->velocityGridLayout->addItem(
+  this->dataPtr->velocityGridLayout->addItem(
       new QSpacerItem(10, 27, QSizePolicy::Expanding,
                       QSizePolicy::Minimum), 0, 0, 2);
 
@@ -673,10 +703,13 @@ void JointControlWidget::LayoutVelocityTab(msgs::Model &_modelMsg)
   QLabel *label = new QLabel("m/s", this);
   label->setFixedHeight(27);
 
-  this->velocityGridLayout->addWidget(label, 0, 2);
-  this->velocityGridLayout->addWidget(new QLabel("P Gain", this), 0, 3);
-  this->velocityGridLayout->addWidget(new QLabel("I Gain", this), 0, 4);
-  this->velocityGridLayout->addWidget(new QLabel("D Gain", this), 0, 5);
+  this->dataPtr->velocityGridLayout->addWidget(label, 0, 2);
+  this->dataPtr->velocityGridLayout->addWidget(new QLabel(
+      "P Gain", this), 0, 3);
+  this->dataPtr->velocityGridLayout->addWidget(new QLabel(
+      "I Gain", this), 0, 4);
+  this->dataPtr->velocityGridLayout->addWidget(new QLabel(
+      "D Gain", this), 0, 5);
 
   int i = 0;
   for (; i < _modelMsg.joint_size(); ++i)
@@ -688,16 +721,16 @@ void JointControlWidget::LayoutVelocityTab(msgs::Model &_modelMsg)
     std::string shortName = jointName.substr(modelNameIndex,
                             jointName.size() - modelNameIndex);
 
-    this->velocityGridLayout->addWidget(
+    this->dataPtr->velocityGridLayout->addWidget(
         new QLabel(tr(shortName.c_str())), i+1, 0);
-    this->velocityGridLayout->addItem(
+    this->dataPtr->velocityGridLayout->addItem(
         new QSpacerItem(10, 20, QSizePolicy::Expanding,
                         QSizePolicy::Minimum), i+1, 1);
 
-    JointPIDVelControl *slider =
-      new JointPIDVelControl(jointName, this->velocityGridLayout, this, i+1);
+    JointPIDVelControl *slider = new JointPIDVelControl(
+        jointName, this->dataPtr->velocityGridLayout, this, i+1);
 
-    this->pidVelSliders[jointName] = slider;
+    this->dataPtr->pidVelSliders[jointName] = slider;
     connect(slider, SIGNAL(changed(double, const std::string &)),
             this, SLOT(OnPIDVelChanged(double, const std::string &)));
     connect(slider, SIGNAL(pChanged(double, const std::string &)),
@@ -709,8 +742,8 @@ void JointControlWidget::LayoutVelocityTab(msgs::Model &_modelMsg)
   }
 
   // Add a space at the bottom of the grid layout to consume extra space.
-  this->velocityGridLayout->addItem(
+  this->dataPtr->velocityGridLayout->addItem(
       new QSpacerItem(10, 20, QSizePolicy::Expanding,
                       QSizePolicy::Expanding), i+1, 0);
-  this->velocityGridLayout->setRowStretch(i+1, 2);
+  this->dataPtr->velocityGridLayout->setRowStretch(i+1, 2);
 }
