@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -128,8 +128,8 @@ void OculusWindow::AttachCameraToVisual()
 
   double pitch = atan2(-delta.z, sqrt(delta.x*delta.x + delta.y*delta.y));
 
-  this->oculusCamera->SetWorldPose(math::Pose(
-        camPos, math::Vector3(0, pitch, yaw)));
+  this->oculusCamera->SetWorldPose(ignition::math::Pose3d(camPos.Ign(),
+      ignition::math::Quaterniond(0.0, pitch, yaw)));
 }
 
 /////////////////////////////////////////////////
