@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,38 +15,46 @@
  *
 */
 
-#include "gazebo/math/Angle.hh"
+#include <ignition/math/Angle.hh>
+
+#include "gazebo/gui/Conversions.hh"
 #include "gazebo/gui/building/BuildingEditorWidget.hh"
 #include "gazebo/gui/building/GrabberHandle.hh"
-#include "gazebo/gui/building/RotateHandle.hh"
-#include "gazebo/gui/building/EditorItem.hh"
+#include "gazebo/gui/building/MeasureItem.hh"
 #include "gazebo/gui/building/RectItem.hh"
+#include "gazebo/gui/building/RectItemPrivate.hh"
+#include "gazebo/gui/building/RotateHandle.hh"
+#include "gazebo/gui/building/WallSegmentItem.hh"
 
 using namespace gazebo;
 using namespace gui;
 
 /////////////////////////////////////////////////
-RectItem::RectItem()
+RectItem::RectItem() : EditorItem(), dataPtr(new RectItemPrivate())
 {
   this->editorType = "Rect";
 
   this->width = 100;
   this->height = 100;
+  this->highlighted = true;
 
   this->drawingOriginX = 0;
   this->drawingOriginY = 0;
 
+  this->dataPtr->positionOnWall = 0;
+  this->dataPtr->angleOnWall = 0;
+
   this->drawingWidth = this->width;
   this->drawingHeight = this->height;
 
-  this->borderColor = Qt::black;
+  this->borderColor = common::Color::Black;
 
   for (int i = 0; i < 8; ++i)
   {
     GrabberHandle *grabber = new GrabberHandle(this, i);
-    this->grabbers.push_back(grabber);
+    this->dataPtr->grabbers.push_back(grabber);
   }
-  this->rotateHandle = new RotateHandle(this);
+  this->dataPtr->rotateHandle = new RotateHandle(this);
 
   this->setSelected(false);
   this->setFlags(this->flags() | QGraphicsItem::ItemIsSelectable);
@@ -55,10 +63,10 @@ RectItem::RectItem()
   this->UpdateCornerPositions();
   this->setAcceptHoverEvents(true);
 
-  this->cursors.push_back(Qt::SizeFDiagCursor);
-  this->cursors.push_back(Qt::SizeVerCursor);
-  this->cursors.push_back(Qt::SizeBDiagCursor);
-  this->cursors.push_back(Qt::SizeHorCursor);
+  this->dataPtr->cursors.push_back(Qt::SizeFDiagCursor);
+  this->dataPtr->cursors.push_back(Qt::SizeVerCursor);
+  this->dataPtr->cursors.push_back(Qt::SizeBDiagCursor);
+  this->dataPtr->cursors.push_back(Qt::SizeHorCursor);
 
   this->setCursor(Qt::SizeAllCursor);
 
@@ -85,11 +93,18 @@ RectItem::~RectItem()
 {
   for (int i = 0; i < 8; ++i)
   {
-    this->grabbers[i]->setParentItem(NULL);
-    delete this->grabbers[i];
+    this->dataPtr->grabbers[i]->setParentItem(NULL);
+    delete this->dataPtr->grabbers[i];
+  }
+
+  this->dataPtr->rotateHandle->setParentItem(NULL);
+  delete this->dataPtr->rotateHandle;
+
+  if (!this->parentItem() && !this->measures.empty())
+  {
+    delete this->measures[0];
+    delete this->measures[1];
   }
-  this->rotateHandle->setParentItem(NULL);
-  delete this->rotateHandle;
 }
 
 /////////////////////////////////////////////////
@@ -97,9 +112,10 @@ void RectItem::ShowHandles(bool _show)
 {
   for (int i = 0; i < 8; ++i)
   {
-    this->grabbers[i]->setVisible(_show && this->grabbers[i]->isEnabled());
+    this->dataPtr->grabbers[i]->setVisible(_show &&
+        this->dataPtr->grabbers[i]->isEnabled());
   }
-  this->rotateHandle->setVisible(_show);
+  this->dataPtr->rotateHandle->setVisible(_show);
 }
 
 /////////////////////////////////////////////////
@@ -117,26 +133,7 @@ QVariant RectItem::itemChange(GraphicsItemChange _change,
 {
   if (_change == QGraphicsItem::ItemSelectedChange && this->scene())
   {
-    if (_value.toBool())
-    {
-      this->setZValue(zValueSelected);
-      for (int i = 0; i < 8; ++i)
-      {
-        if (this->grabbers[i]->isEnabled())
-          this->grabbers[i]->installSceneEventFilter(this);
-      }
-      this->rotateHandle->installSceneEventFilter(this);
-    }
-    else
-    {
-      this->setZValue(zValueIdle);
-      for (int i = 0; i < 8; ++i)
-      {
-        if (this->grabbers[i]->isEnabled())
-          this->grabbers[i]->removeSceneEventFilter(this);
-      }
-      this->rotateHandle->removeSceneEventFilter(this);
-    }
+    this->SetHighlighted(_value.toBool());
   }
   else if (_change == QGraphicsItem::ItemScenePositionHasChanged
       && this->scene())
@@ -147,6 +144,45 @@ QVariant RectItem::itemChange(GraphicsItemChange _change,
   return QGraphicsItem::itemChange(_change, _value);
 }
 
+/////////////////////////////////////////////////
+void RectItem::SetHighlighted(bool _highlighted)
+{
+  if (_highlighted)
+  {
+    this->setZValue(this->ZValueSelected());
+    WallSegmentItem *wallItem = dynamic_cast<WallSegmentItem *>(
+        this->parentItem());
+    if (wallItem)
+      wallItem->setZValue(wallItem->ZValueSelected());
+
+    for (int i = 0; i < 8; ++i)
+    {
+      if (this->dataPtr->grabbers[i]->isEnabled())
+        this->dataPtr->grabbers[i]->installSceneEventFilter(this);
+    }
+    this->dataPtr->rotateHandle->installSceneEventFilter(this);
+    this->Set3dTransparency(0.0);
+  }
+  else
+  {
+    this->setZValue(this->ZValueIdle());
+    WallSegmentItem *wallItem = dynamic_cast<WallSegmentItem *>(
+        this->parentItem());
+    if (wallItem)
+      wallItem->setZValue(wallItem->ZValueIdle());
+
+    for (int i = 0; i < 8; ++i)
+    {
+      if (this->dataPtr->grabbers[i]->isEnabled())
+        this->dataPtr->grabbers[i]->removeSceneEventFilter(this);
+    }
+    this->dataPtr->rotateHandle->removeSceneEventFilter(this);
+    this->Set3dTransparency(0.4);
+  }
+  this->highlighted = _highlighted;
+  this->UpdateMeasures();
+}
+
 /////////////////////////////////////////////////
 bool RectItem::sceneEventFilter(QGraphicsItem * _watched, QEvent *_event)
 {
@@ -209,7 +245,7 @@ bool RectItem::RotateEventFilter(RotateHandle *_rotate, QEvent *_event)
   if (!mouseEvent)
     return false;
 
-  if (_rotate->GetMouseState()
+  if (_rotate->MouseState()
       == static_cast<int>(QEvent::GraphicsSceneMouseMove))
   {
     QPoint localCenter(this->drawingOriginX, this->drawingOriginY);
@@ -233,7 +269,11 @@ bool RectItem::RotateEventFilter(RotateHandle *_rotate, QEvent *_event)
       if (angle < 90 || angle > 270)
       {
         angle = 180;
-        this->SetRotation(this->GetRotation() + angle);
+        this->SetRotation(this->Rotation() + angle);
+        if (this->AngleOnWall() < 90)
+          this->SetAngleOnWall(180);
+        else
+          this->SetAngleOnWall(0);
       }
     }
     else
@@ -241,7 +281,7 @@ bool RectItem::RotateEventFilter(RotateHandle *_rotate, QEvent *_event)
       QLineF prevLine(center.x(), center.y(),
           mouseEvent->lastScenePos().x(), mouseEvent->lastScenePos().y());
       angle = -prevLine.angleTo(line);
-      this->SetRotation(this->GetRotation() + angle);
+      this->SetRotation(this->Rotation() + angle);
     }
   }
   return true;
@@ -278,7 +318,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
     case QEvent::GraphicsSceneHoverMove:
     {
       double angle = this->rotationAngle
-          - static_cast<int>(rotationAngle/360) * 360;
+          - static_cast<int>(this->rotationAngle/360) * 360;
       double range = 22.5;
       if (angle < 0)
         angle += 360;
@@ -287,24 +327,24 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
           || ((angle <= (180 + range)) && (angle > (180 - range))))
       {
         QApplication::setOverrideCursor(
-            QCursor(this->cursors[_grabber->GetIndex() % 4]));
+            QCursor(this->dataPtr->cursors[_grabber->Index() % 4]));
       }
       else if (((angle <= (360 - range)) && (angle > (270 + range)))
           || ((angle <= (180 - range)) && (angle > (90 + range))))
       {
         QApplication::setOverrideCursor(
-            QCursor(this->cursors[(_grabber->GetIndex() + 3) % 4]));
+            QCursor(this->dataPtr->cursors[(_grabber->Index() + 3) % 4]));
       }
       else if (((angle <= (270 + range)) && (angle > (270 - range)))
           || ((angle <= (90 + range)) && (angle > (90 - range))))
       {
         QApplication::setOverrideCursor(
-            QCursor(this->cursors[(_grabber->GetIndex() + 2) % 4]));
+            QCursor(this->dataPtr->cursors[(_grabber->Index() + 2) % 4]));
       }
       else
       {
         QApplication::setOverrideCursor(
-            QCursor(this->cursors[(_grabber->GetIndex() + 1) % 4]));
+            QCursor(this->dataPtr->cursors[(_grabber->Index() + 1) % 4]));
       }
       return true;
     }
@@ -321,7 +361,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
     return false;
 
 
-  if (_grabber->GetMouseState()
+  if (_grabber->MouseState()
       == static_cast<int>(QEvent::GraphicsSceneMouseMove))
   {
     double xPos = mouseEvent->pos().x();
@@ -333,7 +373,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
 
     int xAxisSign = 0;
     int yAxisSign = 0;
-    switch (_grabber->GetIndex())
+    switch (_grabber->Index())
     {
       // corners
       case 0:
@@ -392,8 +432,8 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
     // if the mouse is being dragged, calculate a new size and also position
     // for resizing the box
 
-    double xMoved = _grabber->GetMouseDownX() - xPos;
-    double yMoved = _grabber->GetMouseDownY() - yPos;
+    double xMoved = _grabber->MouseDownX() - xPos;
+    double yMoved = _grabber->MouseDownY() - yPos;
 
     double newWidth = this->width + (xAxisSign * xMoved);
     if (newWidth < 20)
@@ -411,10 +451,11 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
     deltaWidth *= (-1);
     deltaHeight *= (-1);
 
-    double angle = rotationAngle / 360.0 * (2 * M_PI);
+    double angle = this->rotationAngle / 360.0 * (2 * M_PI);
     double dx = 0;
     double dy = 0;
-    switch (_grabber->GetIndex())
+    auto currentPos = Conversions::Convert(this->pos());
+    switch (_grabber->Index())
     {
       // grabbers
       case 0:
@@ -423,7 +464,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
         dy = cos(-angle) * deltaHeight/2;
         dx += cos(angle) * deltaWidth/2;
         dy += sin(angle) * deltaWidth/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
         break;
       }
       case 2:
@@ -432,7 +473,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
         dy = cos(-angle) * deltaHeight/2;
         dx += -cos(angle) * deltaWidth/2;
         dy += -sin(angle) * deltaWidth/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
         break;
       }
       case 4:
@@ -441,7 +482,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
         dy = -cos(-angle) * deltaHeight/2;
         dx += -cos(angle) * deltaWidth/2;
         dy += -sin(angle) * deltaWidth/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
         break;
       }
       case 6:
@@ -450,7 +491,7 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
         dy = -cos(-angle) * deltaHeight/2;
         dx += cos(angle) * deltaWidth/2;
         dy += sin(angle) * deltaWidth/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
         break;
       }
       // edges
@@ -458,28 +499,64 @@ bool RectItem::GrabberEventFilter(GrabberHandle *_grabber, QEvent *_event)
       {
         dx = sin(-angle) * deltaHeight/2;
         dy = cos(-angle) * deltaHeight/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
         break;
       }
       case 3:
       {
         dx = cos(-angle) * deltaWidth/2;
         dy = -sin(-angle) * deltaWidth/2;
-        this->SetPosition(this->pos() - QPointF(dx, dy));
+        this->SetPosition(currentPos - ignition::math::Vector2d(dx, dy));
+        if (this->parentItem())
+        {
+          WallSegmentItem *wallItem = dynamic_cast<WallSegmentItem *>(
+              this->parentItem());
+          if (wallItem)
+          {
+            if (this->AngleOnWall() < 90)
+            {
+              this->dataPtr->positionOnWall -= deltaWidth /
+                  (2*wallItem->line().length());
+            }
+            else
+            {
+              this->dataPtr->positionOnWall += deltaWidth /
+                  (2*wallItem->line().length());
+            }
+          }
+        }
         break;
       }
       case 5:
       {
         dx = sin(-angle) * deltaHeight/2;
         dy = cos(-angle) * deltaHeight/2;
-        this->SetPosition(this->pos() - QPointF(dx, dy));
+        this->SetPosition(currentPos - ignition::math::Vector2d(dx, dy));
         break;
       }
       case 7:
       {
         dx = cos(angle) * deltaWidth/2;
         dy = sin(angle) * deltaWidth/2;
-        this->SetPosition(this->pos() + QPointF(dx, dy));
+        this->SetPosition(currentPos + ignition::math::Vector2d(dx, dy));
+        if (this->parentItem())
+        {
+          WallSegmentItem *wallItem = dynamic_cast<WallSegmentItem *>(
+              this->parentItem());
+          if (wallItem)
+          {
+            if (this->AngleOnWall() < 90)
+            {
+              this->dataPtr->positionOnWall += deltaWidth /
+                  (2*wallItem->line().length());
+            }
+            else
+            {
+              this->dataPtr->positionOnWall -= deltaWidth /
+                  (2*wallItem->line().length());
+            }
+          }
+        }
         break;
       }
       default:
@@ -500,7 +577,7 @@ void RectItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *_event)
   this->location.setY( (static_cast<int>(this->location.y())
       / this->gridSpace) * this->gridSpace);*/
 
-  this->mousePressPos = QPointF(0, 0);
+  this->dataPtr->mousePressPos = ignition::math::Vector2d::Zero;
   _event->setAccepted(true);
 }
 
@@ -512,7 +589,8 @@ void RectItem::mousePressEvent(QGraphicsSceneMouseEvent *_event)
 
 //  this->setSelected(true);
   QApplication::setOverrideCursor(QCursor(Qt::SizeAllCursor));
-  this->mousePressPos = this->mapFromScene(_event->scenePos());
+  this->dataPtr->mousePressPos =
+      Conversions::Convert(this->mapFromScene(_event->scenePos()));
   _event->setAccepted(true);
 }
 
@@ -524,20 +602,21 @@ void RectItem::mouseMoveEvent(QGraphicsSceneMouseEvent *_event)
 
 //  QPointF delta = _event->scenePos() - _event->lastScenePos();
 //  this->SetPosition(this->scenePos() + delta);
-//  this->location += delta;
-//  this->SetPosition(this->location);
+//  this->dataPtr->location += delta;
+//  this->SetPosition(this->dataPtr->location);
 
   // keep track of mouse press pos for more accurate mouse movements than
   // purely relying on mouse translations because we expect items to rotate
   // arbitrary (snap to parent items) when dragged
-  QPointF trans = this->mapFromScene(_event->scenePos()) - this->mousePressPos;
+  QPointF trans = this->mapFromScene(_event->scenePos()) -
+      Conversions::Convert(this->dataPtr->mousePressPos);
   QPointF rotatedTrans;
-  rotatedTrans.setX(cos(GZ_DTOR(this->rotationAngle))*-trans.x()
-    - sin(GZ_DTOR(this->rotationAngle))*-trans.y());
-  rotatedTrans.setY(sin(GZ_DTOR(this->rotationAngle))*-trans.x()
-    + cos(GZ_DTOR(this->rotationAngle))*-trans.y());
+  rotatedTrans.setX(cos(IGN_DTOR(this->rotationAngle))*-trans.x()
+    - sin(IGN_DTOR(this->rotationAngle))*-trans.y());
+  rotatedTrans.setY(sin(IGN_DTOR(this->rotationAngle))*-trans.x()
+    + cos(IGN_DTOR(this->rotationAngle))*-trans.y());
 
-  this->SetPosition(this->pos() - rotatedTrans);
+  this->SetPosition(Conversions::Convert(this->pos() - rotatedTrans));
 }
 
 /////////////////////////////////////////////////
@@ -559,10 +638,10 @@ void RectItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *_event)
 
   for (int i = 0; i < 8; ++i)
   {
-    if (this->grabbers[i]->isEnabled())
-      this->grabbers[i]->removeSceneEventFilter(this);
+    if (this->dataPtr->grabbers[i]->isEnabled())
+      this->dataPtr->grabbers[i]->removeSceneEventFilter(this);
   }
-  this->rotateHandle->removeSceneEventFilter(this);
+  this->dataPtr->rotateHandle->removeSceneEventFilter(this);
 }
 
 /////////////////////////////////////////////////
@@ -588,45 +667,47 @@ void RectItem::hoverEnterEvent(QGraphicsSceneHoverEvent *_event)
 
   QApplication::setOverrideCursor(QCursor(Qt::SizeAllCursor));
 
-  for (unsigned int i = 0; i < this->grabbers.size(); ++i)
+  for (unsigned int i = 0; i < this->dataPtr->grabbers.size(); ++i)
   {
-    if (this->grabbers[i]->isEnabled())
-      this->grabbers[i]->installSceneEventFilter(this);
+    if (this->dataPtr->grabbers[i]->isEnabled())
+      this->dataPtr->grabbers[i]->installSceneEventFilter(this);
   }
-  this->rotateHandle->installSceneEventFilter(this);
+  this->dataPtr->rotateHandle->installSceneEventFilter(this);
 }
 
 /////////////////////////////////////////////////
 void RectItem::UpdateCornerPositions()
 {
-  int grabberWidth = (this->grabbers[0]->boundingRect().width())/2;
-  int grabberHeight = (this->grabbers[0]->boundingRect().height())/2;
+  int grabberWidth = (this->dataPtr->grabbers[0]->boundingRect().width())/2;
+  int grabberHeight = (this->dataPtr->grabbers[0]->boundingRect().height())/2;
 
-  this->grabbers[0]->setPos(
+  this->dataPtr->grabbers[0]->setPos(
       this->drawingOriginX - this->drawingWidth/2 - grabberWidth,
       this->drawingOriginY - this->drawingHeight/2 - grabberHeight);
-  this->grabbers[2]->setPos(
+  this->dataPtr->grabbers[2]->setPos(
       this->drawingOriginX + this->drawingWidth/2 - grabberWidth,
       this->drawingOriginY - this->drawingHeight/2 - grabberHeight);
-  this->grabbers[4]->setPos(
+  this->dataPtr->grabbers[4]->setPos(
       this->drawingOriginX + this->drawingWidth/2 - grabberWidth,
       this->drawingOriginY + this->drawingHeight/2 - grabberHeight);
-  this->grabbers[6]->setPos(
-      this->drawingOriginX - this->drawingWidth/2 - grabberWidth,
+  this->dataPtr->grabbers[6]->setPos(
+      this->drawingOriginX - this->drawingWidth/2 -grabberWidth,
       this->drawingOriginY + this->drawingHeight/2 - grabberHeight);
 
-  this->grabbers[1]->setPos(this->drawingOriginX - grabberWidth,
+  this->dataPtr->grabbers[1]->setPos(
+      this->drawingOriginX - grabberWidth,
       this->drawingOriginY - this->drawingHeight/2 - grabberHeight);
-  this->grabbers[3]->setPos(
+  this->dataPtr->grabbers[3]->setPos(
       this->drawingOriginX + this->drawingWidth/2 - grabberWidth,
       this->drawingOriginY - grabberHeight);
-  this->grabbers[5]->setPos(this->drawingOriginX - grabberWidth,
+  this->dataPtr->grabbers[5]->setPos(
+      this->drawingOriginX - grabberWidth,
       this->drawingOriginY + this->drawingHeight/2 - grabberHeight);
-  this->grabbers[7]->setPos(
+  this->dataPtr->grabbers[7]->setPos(
       this->drawingOriginX - this->drawingWidth/2 - grabberWidth,
       this->drawingOriginY - grabberHeight);
 
-  this->rotateHandle->setPos(this->drawingOriginX,
+  this->dataPtr->rotateHandle->setPos(this->drawingOriginX,
       this->drawingOriginY - this->drawingHeight/2);
 
   this->SizeChanged();
@@ -658,11 +739,11 @@ void RectItem::SetHeight(int _height)
 }
 
 /////////////////////////////////////////////////
-void RectItem::SetSize(QSize _size)
+void RectItem::SetSize(const ignition::math::Vector2i &_size)
 {
-  this->width = _size.width();
+  this->width = _size.X();
   this->drawingWidth = this->width;
-  this->height = _size.height();
+  this->height = _size.Y();
   this->drawingHeight = this->height;
   this->UpdateCornerPositions();
   this->update();
@@ -672,17 +753,43 @@ void RectItem::SetSize(QSize _size)
 }
 
 /////////////////////////////////////////////////
-double RectItem::GetWidth() const
+double RectItem::Width() const
 {
   return this->drawingWidth;
 }
 
 /////////////////////////////////////////////////
-double RectItem::GetHeight() const
+double RectItem::Height() const
 {
   return this->drawingHeight;
 }
 
+/////////////////////////////////////////////////
+void RectItem::SetPositionOnWall(double _positionOnWall)
+{
+  this->dataPtr->positionOnWall = _positionOnWall;
+  this->UpdateMeasures();
+}
+
+/////////////////////////////////////////////////
+double RectItem::PositionOnWall() const
+{
+  return this->dataPtr->positionOnWall;
+}
+
+/////////////////////////////////////////////////
+void RectItem::SetAngleOnWall(double _angleOnWall)
+{
+  this->dataPtr->angleOnWall = _angleOnWall;
+  this->UpdateMeasures();
+}
+
+/////////////////////////////////////////////////
+double RectItem::AngleOnWall() const
+{
+  return this->dataPtr->angleOnWall;
+}
+
 /////////////////////////////////////////////////
 QRectF RectItem::boundingRect() const
 {
@@ -705,19 +812,20 @@ void RectItem::DrawBoundingBox(QPainter *_painter)
 }
 
 /////////////////////////////////////////////////
-QVector3D RectItem::GetSize() const
+ignition::math::Vector3d RectItem::Size() const
 {
-  return QVector3D(this->width, this->height, 0);
+  return ignition::math::Vector3d(this->width, this->height, 0);
 }
 
 /////////////////////////////////////////////////
-QVector3D RectItem::GetScenePosition() const
+ignition::math::Vector3d RectItem::ScenePosition() const
 {
-  return QVector3D(this->scenePos().x(), this->scenePos().y(), 0);
+  return ignition::math::Vector3d(
+      this->scenePos().x(), this->scenePos().y(), 0);
 }
 
 /////////////////////////////////////////////////
-double RectItem::GetSceneRotation() const
+double RectItem::SceneRotation() const
 {
   return this->rotationAngle;
 }
@@ -734,12 +842,12 @@ void RectItem::paint(QPainter *_painter, const QStyleOptionGraphicsItem *,
       this->drawingOriginY - this->drawingHeight/2);
   QPointF bottomLeft(this->drawingOriginX - this->drawingWidth/2,
       this->drawingOriginY + this->drawingHeight/2);
-  QPointF bottomRight(this->drawingOriginX  + this->drawingWidth/2,
+  QPointF bottomRight(this->drawingOriginX + this->drawingWidth/2,
       this->drawingOriginY + this->drawingHeight/2);
 
   QPen rectPen;
   rectPen.setStyle(Qt::SolidLine);
-  rectPen.setColor(borderColor);
+  rectPen.setColor(Conversions::Convert(this->borderColor));
   _painter->setPen(rectPen);
 
   _painter->drawLine(topLeft, topRight);
@@ -782,9 +890,9 @@ void RectItem::OnDeleteItem()
 }
 
 /////////////////////////////////////////////////
-void RectItem::SetPosition(const QPointF &_pos)
+void RectItem::SetPosition(const ignition::math::Vector2d &_pos)
 {
-  this->SetPosition(_pos.x(), _pos.y());
+  this->SetPosition(_pos.X(), _pos.Y());
 }
 
 /////////////////////////////////////////////////
@@ -804,7 +912,7 @@ void RectItem::SetRotation(double _angle)
 }
 
 /////////////////////////////////////////////////
-double RectItem::GetRotation() const
+double RectItem::Rotation() const
 {
   return this->rotationAngle;
 }
@@ -814,34 +922,120 @@ void RectItem::SizeChanged()
 {
   emit DepthChanged(this->drawingHeight);
   emit WidthChanged(this->drawingWidth);
+  this->UpdateMeasures();
 }
 
 /////////////////////////////////////////////////
 void RectItem::SetResizeFlag(unsigned int _flag)
 {
-  if (this->resizeFlag == _flag)
+  if (this->dataPtr->resizeFlag == _flag)
     return;
 
-  this->resizeFlag = _flag;
+  this->dataPtr->resizeFlag = _flag;
   for (int i = 0; i < 8; ++i)
-    this->grabbers[i]->setEnabled(false);
+    this->dataPtr->grabbers[i]->setEnabled(false);
 
-
-  if (resizeFlag & ITEM_WIDTH)
+  if (this->dataPtr->resizeFlag & ITEM_WIDTH)
   {
-    this->grabbers[3]->setEnabled(true);
-    this->grabbers[7]->setEnabled(true);
+    this->dataPtr->grabbers[3]->setEnabled(true);
+    this->dataPtr->grabbers[7]->setEnabled(true);
   }
-  if (resizeFlag & ITEM_HEIGHT)
+  if (this->dataPtr->resizeFlag & ITEM_HEIGHT)
   {
-    this->grabbers[1]->setEnabled(true);
-    this->grabbers[5]->setEnabled(true);
+    this->dataPtr->grabbers[1]->setEnabled(true);
+    this->dataPtr->grabbers[5]->setEnabled(true);
   }
-  if ((resizeFlag & ITEM_WIDTH) && (resizeFlag & ITEM_HEIGHT))
+  if ((this->dataPtr->resizeFlag & ITEM_WIDTH) &&
+      (this->dataPtr->resizeFlag & ITEM_HEIGHT))
   {
-    this->grabbers[0]->setEnabled(true);
-    this->grabbers[2]->setEnabled(true);
-    this->grabbers[4]->setEnabled(true);
-    this->grabbers[6]->setEnabled(true);
+    this->dataPtr->grabbers[0]->setEnabled(true);
+    this->dataPtr->grabbers[2]->setEnabled(true);
+    this->dataPtr->grabbers[4]->setEnabled(true);
+    this->dataPtr->grabbers[6]->setEnabled(true);
   }
 }
+
+/////////////////////////////////////////////////
+void RectItem::UpdateMeasures()
+{
+  // Only windows and doors can have a wall as parent
+  WallSegmentItem *wallItem = dynamic_cast<WallSegmentItem *>(
+    this->parentItem());
+  if (wallItem == NULL)
+  {
+    for (unsigned int i = 0; i < this->measures.size(); ++i)
+    {
+      this->measures[i]->setVisible(false);
+    }
+    return;
+  }
+
+  if (this->measures.empty())
+  {
+    this->measures.push_back(new MeasureItem(ignition::math::Vector2d(0, 0),
+                                             ignition::math::Vector2d(0, 1)));
+    this->measures.push_back(new MeasureItem(ignition::math::Vector2d(0, 0),
+                                             ignition::math::Vector2d(0, 1)));
+    this->measures[0]->setVisible(false);
+    this->measures[1]->setVisible(false);
+  }
+
+  this->measures[0]->setParentItem(wallItem);
+  this->measures[1]->setParentItem(wallItem);
+  this->measures[0]->setVisible(this->highlighted);
+  this->measures[1]->setVisible(this->highlighted);
+
+  if (this->highlighted)
+  {
+    // Half wall thickness
+    double t = wallItem->Thickness()/2;
+    // Distance in px between wall line and measure line
+    double d = 20 + t;
+    // Half the RectItem's length
+    double w = this->drawingWidth/2;
+    // This item's angle on the scene
+    double angle = IGN_DTOR(this->rotationAngle);
+    // Free vector of t on wall direction, for the extremes
+    ignition::math::Vector2d tVec(t*qCos(angle), t*qSin(angle));
+    // Free vector of d perpendicular to the wall
+    ignition::math::Vector2d dVec(d*cos(angle+M_PI/2.0),
+                                  d*sin(angle+M_PI/2.0));
+
+    auto p1wall = wallItem->StartPoint();
+    auto p2wall = wallItem->EndPoint();
+    ignition::math::Vector2d p1rect(this->scenePos().x()-w*qCos(angle),
+                                    this->scenePos().y()-w*qSin(angle));
+    ignition::math::Vector2d p2rect(this->scenePos().x()+w*qCos(angle),
+                                    this->scenePos().y()+w*qSin(angle));
+
+    auto extreme1 = p1wall;
+    auto extreme2 = p2wall;
+
+    // Swap extremes if item is flipped on wall
+    if (this->AngleOnWall() > 90)
+    {
+      extreme1 = p2wall;
+      extreme2 = p1wall;
+    }
+
+    // Measure 0, from extreme 1 to RectItem's start point
+    this->measures[0]->SetStartPoint(extreme1 - tVec - dVec);
+    this->measures[0]->SetEndPoint(p1rect - dVec);
+    // Measure 1, from RectItem's end point to extreme 2
+    this->measures[1]->SetStartPoint(p2rect - dVec);
+    this->measures[1]->SetEndPoint(extreme2 + tVec - dVec);
+
+    this->measures[0]->SetValue(
+        (this->measures[0]->line().length())*this->itemScale);
+    this->measures[1]->SetValue(
+        (this->measures[1]->line().length())*this->itemScale);
+  }
+}
+
+/////////////////////////////////////////////////
+void RectItem::DetachFromParent()
+{
+  this->setParentItem(NULL);
+  for (unsigned int i = 0; i < this->measures.size(); i++)
+    this->measures[i]->setParentItem(NULL);
+}
