@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,16 +14,15 @@
  * limitations under the License.
  *
  */
-
 #ifdef _WIN32
   // Ensure that Winsock2.h is included before Windows.h, which can get
   // pulled in by anybody (e.g., Boost).
   #include <Winsock2.h>
 #endif
 
-#include "gazebo/transport/TransportIface.hh"
+#include <memory>
+
 #include "gazebo/transport/Node.hh"
-#include "gazebo/transport/Publisher.hh"
 
 #include "gazebo/common/Image.hh"
 
@@ -44,15 +43,13 @@ ImagesView::ImagesView(QWidget *_parent)
   this->setWindowTitle(tr("Gazebo: Images View"));
 
   // Create the layout and frame for images
-  // {
-  this->dataPtr->frameLayout = new QGridLayout;
-  this->dataPtr->frameLayout->setSizeConstraint(QLayout::SetMinimumSize);
+  std::unique_ptr<QGridLayout> frameLayout(new QGridLayout);
+  frameLayout->setSizeConstraint(QLayout::SetMinimumSize);
 
   this->frame->setObjectName("blackBorderFrame");
-  this->frame->setLayout(this->dataPtr->frameLayout);
+  this->frame->setLayout(frameLayout.release());
   this->frame->setMinimumHeight(240);
   this->frame->setMinimumWidth(320);
-  // }
 
   this->dataPtr->clearImages = false;
 }
@@ -60,32 +57,33 @@ ImagesView::ImagesView(QWidget *_parent)
 /////////////////////////////////////////////////
 ImagesView::~ImagesView()
 {
+  this->sub.reset();
+
   delete this->dataPtr;
   this->dataPtr = NULL;
-
-  this->sub.reset();
 }
 
 /////////////////////////////////////////////////
 void ImagesView::UpdateImpl()
 {
-  boost::mutex::scoped_lock lock(this->dataPtr->mutex);
-  std::vector<ImageFrame *>::iterator imageIter = this->dataPtr->images.begin();
+  std::lock_guard<std::mutex> lock(this->dataPtr->mutex);
 
   // Clear out the images if the flag is set.
   if (this->dataPtr->clearImages)
   {
-    // Remove all the images, and delete them
-    for (; imageIter != this->dataPtr->images.end(); )
+    // Clear previous layout
+    auto oldLayout = this->frame->layout();
+    if (oldLayout)
     {
-      (*imageIter)->hide();
-      this->dataPtr->frameLayout->removeWidget(*imageIter);
-      delete *imageIter;
-      imageIter = this->dataPtr->images.erase(imageIter);
+      // Let Qt delete the widgets. Give ownership of all widgets to an object
+      // which will be out of scope.
+      QWidget().setLayout(oldLayout);
     }
 
-    // Clear the lists
-    this->dataPtr->images.clear();
+    // Create new layout
+    std::unique_ptr<QGridLayout> newLayout(new QGridLayout);
+    newLayout->setSizeConstraint(QLayout::SetMinimumSize);
+    this->frame->setLayout(newLayout.release());
 
     // Make sure to adjust the size of the widget
     this->frame->adjustSize();
@@ -109,7 +107,7 @@ void ImagesView::UpdateImpl()
 /////////////////////////////////////////////////
 void ImagesView::SetTopic(const std::string &_topicName)
 {
-  boost::mutex::scoped_lock lock(this->dataPtr->mutex);
+  std::lock_guard<std::mutex> lock(this->dataPtr->mutex);
 
   // Tell the widget to clear the images
   this->dataPtr->clearImages = true;
@@ -117,30 +115,34 @@ void ImagesView::SetTopic(const std::string &_topicName)
   TopicView::SetTopic(_topicName);
 
   // Subscribe to the new topic.
-  this->sub = this->node->Subscribe(_topicName, &ImagesView::OnImages, this);
+  if (this->node)
+    this->sub = this->node->Subscribe(_topicName, &ImagesView::OnImages, this);
 }
 
 /////////////////////////////////////////////////
 void ImagesView::AddImage(int _width, int _height)
 {
-  ImageFrame *imageFrame = new ImageFrame(this);
+  std::unique_ptr<ImageFrame> imageFrame(new ImageFrame(NULL));
   imageFrame->setBaseSize(_width, _height);
   imageFrame->setMinimumSize(320, 240);
   imageFrame->show();
-  this->dataPtr->images.push_back(imageFrame);
 
   // Add the lable to the correct row and column
-  this->dataPtr->frameLayout->addWidget(imageFrame,
-      (this->dataPtr->images.size()-1) / 2,
-      (this->dataPtr->images.size()-1) % 2);
+  auto frameLayout = qobject_cast<QGridLayout *>(this->frame->layout());
+  if (!frameLayout)
+    return;
+
+  frameLayout->addWidget(imageFrame.release(),
+      (frameLayout->count()) / 2,
+      (frameLayout->count()) % 2);
 }
 
 /////////////////////////////////////////////////
 void ImagesView::OnImages(ConstImagesStampedPtr &_msg)
 {
   // Only use a try lock so that we don't block the node thread.
-  boost::mutex::scoped_try_lock lock(this->dataPtr->mutex);
-  if (!lock)
+  std::unique_lock<std::mutex> lock(this->dataPtr->mutex, std::try_to_lock);
+  if (!lock.owns_lock())
       return;
 
   if (this->dataPtr->clearImages)
@@ -153,14 +155,22 @@ void ImagesView::OnImages(ConstImagesStampedPtr &_msg)
   {
     dataSize += _msg->image(i).data().size();
 
-    if (i >= static_cast<int>(this->dataPtr->images.size()))
+    if (i >= this->frame->layout()->count())
     {
       this->dataPtr->addImage.push_back(std::make_pair(_msg->image(i).width(),
                                               _msg->image(i).height()));
       continue;
     }
 
-    this->dataPtr->images[i]->OnImage(_msg->image(i));
+    auto frameLayout = qobject_cast<QGridLayout *>(this->frame->layout());
+    if (!frameLayout)
+      continue;
+
+    auto imageFrame = qobject_cast<ImageFrame *>(
+        frameLayout->itemAtPosition(i / 2, i % 2)->widget());
+
+    if (imageFrame)
+        imageFrame->OnImage(_msg->image(i));
   }
 
   // Update the Hz and Bandwidth info
