@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -55,22 +55,7 @@ Base::Base(BasePtr _parent)
 //////////////////////////////////////////////////
 Base::~Base()
 {
-  // remove self as a child of the parent
-  if (this->parent)
-    this->parent->RemoveChild(this->id);
-
-  this->SetParent(BasePtr());
-
-  for (Base_V::iterator iter = this->children.begin();
-       iter != this->children.end(); ++iter)
-  {
-    if (*iter)
-      (*iter)->SetParent(BasePtr());
-  }
-  this->children.clear();
-  if (this->sdf)
-    this->sdf->Reset();
-  this->sdf.reset();
+  this->Fini();
 }
 
 //////////////////////////////////////////////////
@@ -106,16 +91,26 @@ void Base::UpdateParameters(sdf::ElementPtr _sdf)
 //////////////////////////////////////////////////
 void Base::Fini()
 {
-  Base_V::iterator iter;
+  // Remove self as a child of the parent
+  if (this->parent)
+  {
+    auto temp = this->parent;
+    this->parent.reset();
 
-  for (iter = this->children.begin(); iter != this->children.end(); ++iter)
-    if (*iter)
-      (*iter)->Fini();
+    temp->RemoveChild(this->id);
+  }
 
+  // Also destroy all children.
+  while (!this->children.empty())
+  {
+    auto child = this->children.front();
+    this->RemoveChild(child);
+  }
   this->children.clear();
 
+  this->sdf.reset();
+
   this->world.reset();
-  this->parent.reset();
 }
 
 //////////////////////////////////////////////////
@@ -195,22 +190,17 @@ void Base::AddChild(BasePtr _child)
     gzthrow("Cannot add a null _child to an entity");
 
   // Add this _child to our list
-  this->children.push_back(_child);
+  if (std::find(this->children.begin(), this->children.end(), _child)
+      == this->children.end())
+  {
+    this->children.push_back(_child);
+  }
 }
 
 //////////////////////////////////////////////////
 void Base::RemoveChild(unsigned int _id)
 {
-  Base_V::iterator iter;
-  for (iter = this->children.begin(); iter != this->children.end(); ++iter)
-  {
-    if ((*iter)->GetId() == _id)
-    {
-      (*iter)->Fini();
-      this->children.erase(iter);
-      break;
-    }
-  }
+  this->RemoveChild(this->GetById(_id));
 }
 
 //////////////////////////////////////////////////
@@ -244,19 +234,23 @@ BasePtr Base::GetChild(const std::string &_name)
 //////////////////////////////////////////////////
 void Base::RemoveChild(const std::string &_name)
 {
-  Base_V::iterator iter;
+  this->RemoveChild(this->GetByName(_name));
+}
 
-  for (iter = this->children.begin(); iter != this->children.end(); ++iter)
-  {
-    if ((*iter)->GetScopedName() == _name)
-      break;
-  }
+//////////////////////////////////////////////////
+void Base::RemoveChild(physics::BasePtr _child)
+{
+  if (!_child)
+    return;
 
-  if (iter != this->children.end())
-  {
-    (*iter)->Fini();
-    this->children.erase(iter);
-  }
+  // Fini
+  _child->SetParent(nullptr);
+  _child->Fini();
+
+  // Remove from vector if still there
+  this->children.erase(std::remove(this->children.begin(),
+                                   this->children.end(), _child),
+                                   this->children.end());
 }
 
 //////////////////////////////////////////////////
@@ -302,7 +296,7 @@ BasePtr Base::GetByName(const std::string &_name)
 //////////////////////////////////////////////////
 std::string Base::GetScopedName(bool _prependWorldName) const
 {
-  if (_prependWorldName)
+  if (_prependWorldName && this->world)
     return this->world->GetName() + "::" + this->scopedName;
   else
     return this->scopedName;
