@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@ using namespace physics;
 Entity::Entity(BasePtr _parent)
   : Base(_parent)
 {
+  this->isStatic = false;
   this->isCanonicalLink = false;
   this->node = transport::NodePtr(new transport::Node());
   this->AddType(ENTITY);
@@ -80,22 +81,13 @@ Entity::Entity(BasePtr _parent)
 
   this->setWorldPoseFunc = &Entity::SetWorldPoseDefault;
 
-  this->scale = math::Vector3::One;
+  this->scale = ignition::math::Vector3d::One;
 }
 
 //////////////////////////////////////////////////
 Entity::~Entity()
 {
-  // TODO: put this back in
-  // this->GetWorld()->GetPhysicsEngine()->RemoveEntity(this);
-
-  delete this->visualMsg;
-  this->visualMsg = NULL;
-
-  this->visPub.reset();
-  this->requestPub.reset();
-  this->poseSub.reset();
-  this->node.reset();
+  this->Fini();
 }
 
 //////////////////////////////////////////////////
@@ -337,29 +329,44 @@ void Entity::SetWorldPoseModel(const math::Pose &_pose, bool _notify,
     {
       EntityPtr entity = boost::static_pointer_cast<Entity>(*iter);
 
-      if (entity->IsCanonicalLink())
-        entity->worldPose = (entity->initialRelativePose + _pose);
+      if (entity->HasType(LINK))
+      {
+        if (entity->IsCanonicalLink())
+          entity->worldPose = (entity->initialRelativePose + _pose);
+        else
+        {
+          entity->worldPose = ((entity->worldPose - oldModelWorldPose) + _pose);
+          if (_publish)
+            entity->PublishPose();
+        }
+
+        if (_notify)
+          entity->UpdatePhysicsPose(false);
+
+        // Tell collisions that their current world pose is dirty (needs
+        // updating). We set a dirty flag instead of directly updating the
+        // value to improve performance.
+        for (Base_V::iterator iterC = (*iter)->children.begin();
+             iterC != (*iter)->children.end(); ++iterC)
+        {
+          if ((*iterC)->HasType(COLLISION))
+          {
+            CollisionPtr entityC =
+                boost::static_pointer_cast<Collision>(*iterC);
+            entityC->SetWorldPoseDirty();
+          }
+        }
+      }
+      else if (entity->HasType(MODEL))
+      {
+        // set pose of nested models
+        entity->SetWorldPoseModel(
+            (entity->worldPose - oldModelWorldPose) + _pose, _notify, _publish);
+      }
       else
       {
-        entity->worldPose = ((entity->worldPose - oldModelWorldPose) + _pose);
-        if (_publish)
-          entity->PublishPose();
-      }
-
-      if (_notify)
-        entity->UpdatePhysicsPose(false);
-
-      // Tell collisions that their current world pose is dirty (needs
-      // updating). We set a dirty flag instead of directly updating the
-      // value to improve performance.
-      for (Base_V::iterator iterC = (*iter)->children.begin();
-           iterC != (*iter)->children.end(); ++iterC)
-      {
-        if ((*iterC)->HasType(COLLISION))
-        {
-          CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
-          entityC->SetWorldPoseDirty();
-        }
+        gzerr << "SetWorldPoseModel error: unknown type of entity in Model."
+          << std::endl;
       }
     }
   }
@@ -375,37 +382,51 @@ void Entity::SetWorldPoseCanonicalLink(const math::Pose &_pose, bool _notify,
   if (_notify)
     this->UpdatePhysicsPose(true);
 
-  // also update parent model's pose
-  if (this->parentEntity->HasType(MODEL))
+  if (!this->parentEntity->HasType(MODEL))
+  {
+    gzerr << "SetWorldPose for Canonical Body [" << this->GetName()
+        << "] but parent[" << this->parentEntity->GetName()
+        << "] is not a MODEL!" << std::endl;
+    return;
+  }
+
+  EntityPtr parentEnt = this->parentEntity;
+  ignition::math::Pose3d relativePose = this->initialRelativePose.Ign();
+  math::Pose updatePose = _pose;
+
+  // recursively update parent model pose based on new canonical link pose
+  while (parentEnt && parentEnt->HasType(MODEL))
   {
     // setting parent Model world pose from canonical link world pose
     // where _pose is the canonical link's world pose
-    this->parentEntity->worldPose = (-this->initialRelativePose) + _pose;
+    parentEnt->worldPose = math::Pose(-relativePose) + updatePose;
 
-    this->parentEntity->worldPose.Correct();
+    parentEnt->worldPose.Correct();
 
     if (_notify)
-      this->parentEntity->UpdatePhysicsPose(false);
+      parentEnt->UpdatePhysicsPose(false);
 
     if (_publish)
       this->parentEntity->PublishPose();
 
-    // Tell collisions that their current world pose is dirty (needs
-    // updating). We set a dirty flag instead of directly updating the
-    // value to improve performance.
-    for (Base_V::iterator iterC = this->children.begin();
-        iterC != this->children.end(); ++iterC)
+    updatePose = parentEnt->worldPose;
+    relativePose = parentEnt->GetInitialRelativePose().Ign();
+
+    parentEnt = boost::dynamic_pointer_cast<Entity>(parentEnt->GetParent());
+  }
+
+  // Tell collisions that their current world pose is dirty (needs
+  // updating). We set a dirty flag instead of directly updating the
+  // value to improve performance.
+  for (Base_V::iterator iterC = this->children.begin();
+      iterC != this->children.end(); ++iterC)
+  {
+    if ((*iterC)->HasType(COLLISION))
     {
-      if ((*iterC)->HasType(COLLISION))
-      {
-        CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
-        entityC->SetWorldPoseDirty();
-      }
+      CollisionPtr entityC = boost::static_pointer_cast<Collision>(*iterC);
+      entityC->SetWorldPoseDirty();
     }
   }
-  else
-    gzerr << "SWP for CB[" << this->GetName() << "] but parent["
-      << this->parentEntity->GetName() << "] is not a MODEL!\n";
 }
 
 //////////////////////////////////////////////////
@@ -555,27 +576,44 @@ void Entity::OnPoseMsg(ConstPosePtr &_msg)
 //////////////////////////////////////////////////
 void Entity::Fini()
 {
+  // TODO: put this back in
+  // this->GetWorld()->GetPhysicsEngine()->RemoveEntity(this);
+
   if (this->requestPub)
   {
-    msgs::Request *msg = msgs::CreateRequest("entity_delete",
-        this->GetScopedName());
+    auto msg = msgs::CreateRequest("entity_delete", this->GetScopedName());
     this->requestPub->Publish(*msg, true);
   }
 
-  this->parentEntity.reset();
-  Base::Fini();
+  // Clean transport
+  {
+    this->posePub.reset();
+    this->requestPub.reset();
+    this->visPub.reset();
 
+    this->poseSub.reset();
+
+    if (this->node)
+      this->node->Fini();
+    this->node.reset();
+  }
+
+  this->animationConnection.reset();
   this->connections.clear();
-  this->node->Fini();
+
+  if (this->visualMsg)
+    delete this->visualMsg;
+  this->visualMsg = NULL;
+
+  this->parentEntity.reset();
+
+  Base::Fini();
 }
 
 //////////////////////////////////////////////////
 void Entity::Reset()
 {
-  if (this->HasType(Base::MODEL))
-    this->SetWorldPose(this->initialRelativePose);
-  else
-    this->SetRelativePose(this->initialRelativePose);
+  this->SetRelativePose(this->initialRelativePose);
 }
 
 //////////////////////////////////////////////////
