@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,9 +24,6 @@
 #include "gazebo/transport/TransportIface.hh"
 #include "gazebo/transport/Publisher.hh"
 
-#include "gazebo/sensors/Sensor.hh"
-#include "gazebo/sensors/SensorsIface.hh"
-
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Events.hh"
@@ -82,12 +79,7 @@ Joint::Joint(BasePtr _parent)
 //////////////////////////////////////////////////
 Joint::~Joint()
 {
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-      iter != this->sensors.end(); ++iter)
-  {
-    sensors::remove_sensor(*iter);
-  }
-  this->sensors.clear();
+  this->Fini();
 }
 
 //////////////////////////////////////////////////
@@ -215,6 +207,12 @@ void Joint::Load(sdf::ElementPtr _sdf)
   if (this->model)
   {
     this->childLink = this->model->GetLink(childName);
+    if (!this->childLink)
+    {
+      // need to do this if child link belongs to another model
+      this->childLink = boost::dynamic_pointer_cast<Link>(
+          this->GetWorld()->GetByName(childName));
+    }
     this->parentLink = this->model->GetLink(parentName);
   }
   else
@@ -226,11 +224,77 @@ void Joint::Load(sdf::ElementPtr _sdf)
         this->GetWorld()->GetByName(parentName));
   }
 
+  // Link might not have been found because it is on another model
+  // or because the model name has been changed, e.g. spawning the same model
+  // twice will result in some suffix appended to the model name
+  // First try to find the link with different scopes.
   if (!this->parentLink && parentName != std::string("world"))
-    gzthrow("Couldn't Find Parent Link[" + parentName + "]");
+  {
+    BasePtr parentModel = this->model;
+    while (!this->parentLink && parentModel && parentModel->HasType(MODEL))
+    {
+      std::string scopedParentName =
+          parentModel->GetScopedName() + "::" + parentName;
+
+      this->parentLink = boost::dynamic_pointer_cast<Link>(
+          this->GetWorld()->GetByName(scopedParentName));
+
+      parentModel = parentModel->GetParent();
+    }
+    if (!this->parentLink)
+    {
+      std::string parentNameThisModel;
+      auto doubleColon = parentName.find("::");
+      if (doubleColon != std::string::npos)
+      {
+        parentNameThisModel = parentName.substr(doubleColon);
+      }
+      else
+      {
+        parentNameThisModel = "::" + parentName;
+      }
+      parentNameThisModel = parentModel->GetName() + parentNameThisModel;
+
+      this->parentLink = boost::dynamic_pointer_cast<Link>(
+          this->GetWorld()->GetByName(parentNameThisModel));
+    }
+    if (!this->parentLink)
+      gzthrow("Couldn't Find Parent Link[" + parentName + "]");
+  }
 
   if (!this->childLink && childName != std::string("world"))
-    gzthrow("Couldn't Find Child Link[" + childName  + "]");
+  {
+    BasePtr parentModel = this->model;
+
+    while (!this->childLink && parentModel && parentModel->HasType(MODEL))
+    {
+      std::string scopedChildName =
+          parentModel->GetScopedName() + "::" + childName;
+      this->childLink = boost::dynamic_pointer_cast<Link>(
+          this->GetWorld()->GetByName(scopedChildName));
+
+      parentModel = parentModel->GetParent();
+    }
+    if (!this->childLink)
+    {
+      std::string childNameThisModel;
+      auto doubleColon = childName.find("::");
+      if (doubleColon != std::string::npos)
+      {
+        childNameThisModel = childName.substr(doubleColon);
+      }
+      else
+      {
+        childNameThisModel = "::" + childName;
+      }
+      childNameThisModel = parentModel->GetName() + childNameThisModel;
+
+      this->childLink = boost::dynamic_pointer_cast<Link>(
+          this->GetWorld()->GetByName(childNameThisModel));
+    }
+    if (!this->childLink)
+      gzthrow("Couldn't Find Child Link[" + childName  + "]");
+  }
 
   this->LoadImpl(_sdf->Get<math::Pose>("pose"));
 }
@@ -270,9 +334,14 @@ void Joint::LoadImpl(const math::Pose &_pose)
       /// other sensors. We should make this more generic.
       if (sensorElem->Get<std::string>("type") == "force_torque")
       {
-        std::string sensorName =
-          sensors::create_sensor(sensorElem, this->GetWorld()->GetName(),
-              this->GetScopedName(), this->GetId());
+        // This must match the implementation in Sensors::GetScopedName
+        std::string sensorName = this->GetScopedName(true) + "::" +
+          sensorElem->Get<std::string>("name");
+
+        // Tell the sensor library to create a sensor.
+        event::Events::createSensor(sensorElem,
+            this->GetWorld()->GetName(), this->GetScopedName(), this->GetId());
+
         this->sensors.push_back(sensorName);
       }
       else
@@ -341,13 +410,21 @@ void Joint::Init()
 //////////////////////////////////////////////////
 void Joint::Fini()
 {
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-      iter != this->sensors.end(); ++iter)
+  this->applyDamping.reset();
+
+  // Remove all the sensors attached to the joint
+  for (auto const &sensor : this->sensors)
   {
-    sensors::remove_sensor(*iter);
+    event::Events::removeSensor(sensor);
   }
   this->sensors.clear();
 
+  this->anchorLink.reset();
+  this->childLink.reset();
+  this->parentLink.reset();
+  this->model.reset();
+  this->sdfJoint.reset();
+
   Base::Fini();
 }
 
@@ -422,6 +499,7 @@ void Joint::Update()
 void Joint::UpdateParameters(sdf::ElementPtr _sdf)
 {
   Base::UpdateParameters(_sdf);
+  /// \todo Update joint specific parameters. Issue #1954
 }
 
 //////////////////////////////////////////////////
@@ -582,18 +660,17 @@ void Joint::FillMsg(msgs::Joint &_msg)
     _msg.set_parent_id(0);
   }
 
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-       iter != this->sensors.end(); ++iter)
+  // Add in the sensor data.
+  if (this->sdf->HasElement("sensor"))
   {
-    sensors::SensorPtr sensor = sensors::get_sensor(*iter);
-    if (sensor)
+    sdf::ElementPtr sensorElem = this->sdf->GetElement("sensor");
+    while (sensorElem)
     {
-      msgs::Sensor *sensorMsg =_msg.add_sensor();
-      sensor->FillMsg(*sensorMsg);
-    }
-    else
-    {
-      gzlog << "Joint::FillMsg: sensor [" << *iter << "] not found.\n";
+      msgs::Sensor *msg = _msg.add_sensor();
+      msg->CopyFrom(msgs::SensorFromSDF(sensorElem));
+      msg->set_parent(this->GetScopedName());
+      msg->set_parent_id(this->GetId());
+      sensorElem = sensorElem->GetNextElement("sensor");
     }
   }
 }
@@ -625,12 +702,6 @@ bool Joint::SetLowStop(unsigned int _index, const math::Angle &_angle)
   return true;
 }
 
-//////////////////////////////////////////////////
-void Joint::SetAngle(unsigned int _index, math::Angle _angle)
-{
-  this->SetPosition(_index, _angle.Radian());
-}
-
 //////////////////////////////////////////////////
 bool Joint::SetPosition(unsigned int /*_index*/, double _position)
 {
