@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/bind.hpp>
 #include <sstream>
+#include <functional>
 
 #include "gazebo/msgs/msgs.hh"
 
@@ -37,9 +38,7 @@
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Assert.hh"
-
-#include "gazebo/sensors/SensorsIface.hh"
-#include "gazebo/sensors/Sensor.hh"
+#include "gazebo/common/Battery.hh"
 
 #include "gazebo/physics/PhysicsIface.hh"
 #include "gazebo/physics/Model.hh"
@@ -67,57 +66,7 @@ Link::Link(EntityPtr _parent)
 //////////////////////////////////////////////////
 Link::~Link()
 {
-  this->attachedModels.clear();
-
-  for (Visuals_M::iterator iter = this->visuals.begin();
-      iter != this->visuals.end(); ++iter)
-  {
-    msgs::Visual msg;
-    msg.set_name(iter->second.name());
-    msg.set_id(iter->second.id());
-    if (this->parent)
-    {
-      msg.set_parent_name(this->parent->GetScopedName());
-      msg.set_parent_id(this->parent->GetId());
-    }
-    else
-    {
-      msg.set_parent_name("");
-      msg.set_parent_id(0);
-    }
-    msg.set_delete_me(true);
-    this->visPub->Publish(msg);
-  }
-  this->visuals.clear();
-
-  if (this->cgVisuals.size() > 0)
-  {
-    for (unsigned int i = 0; i < this->cgVisuals.size(); i++)
-    {
-      msgs::Visual msg;
-      msg.set_name(this->cgVisuals[i]);
-      if (this->parent)
-        msg.set_parent_name(this->parent->GetScopedName());
-      else
-        msg.set_parent_name("");
-      msg.set_delete_me(true);
-      this->visPub->Publish(msg);
-    }
-    this->cgVisuals.clear();
-  }
-
-  this->visPub.reset();
-  this->sensors.clear();
-
-  this->requestPub.reset();
-  this->dataPub.reset();
-  this->wrenchSub.reset();
-  this->connections.clear();
-
-  delete this->publishDataMutex;
-  this->publishDataMutex = NULL;
-
-  this->collisions.clear();
+  this->Fini();
 }
 
 //////////////////////////////////////////////////
@@ -136,7 +85,7 @@ void Link::Load(sdf::ElementPtr _sdf)
     this->SetSelfCollide(this->GetModel()->GetSelfCollide());
   }
   this->sdf->GetElement("self_collide")->GetValue()->SetUpdateFunc(
-      boost::bind(&Link::GetSelfCollide, this));
+      std::bind(&Link::GetSelfCollide, this));
 
   // Parse visuals from SDF
   this->ParseVisuals();
@@ -167,9 +116,14 @@ void Link::Load(sdf::ElementPtr _sdf)
       }
       else if (sensorElem->Get<std::string>("type") != "__default__")
       {
-        std::string sensorName =
-          sensors::create_sensor(sensorElem, this->GetWorld()->GetName(),
-              this->GetScopedName(), this->GetId());
+        // This must match the implementation in Sensors::GetScopedName
+        std::string sensorName = this->GetScopedName(true) + "::" +
+          sensorElem->Get<std::string>("name");
+
+        // Tell the sensor library to create a sensor.
+        event::Events::createSensor(sensorElem,
+            this->GetWorld()->GetName(), this->GetScopedName(), this->GetId());
+
         this->sensors.push_back(sensorName);
       }
       sensorElem = sensorElem->GetNextElement("sensor");
@@ -193,7 +147,7 @@ void Link::Load(sdf::ElementPtr _sdf)
       util::OpenALSourcePtr source = util::OpenAL::Instance()->CreateSource(
           audioElem);
 
-      std::vector<std::string> names = source->GetCollisionNames();
+      std::vector<std::string> names = source->CollisionNames();
       std::copy(names.begin(), names.end(), std::back_inserter(collisionNames));
 
       audioElem = audioElem->GetNextElement("audio_source");
@@ -223,6 +177,16 @@ void Link::Load(sdf::ElementPtr _sdf)
   }
 #endif
 
+  if (this->sdf->HasElement("battery"))
+  {
+    sdf::ElementPtr batteryElem = this->sdf->GetElement("battery");
+    while (batteryElem)
+    {
+      this->LoadBattery(batteryElem);
+      batteryElem = batteryElem->GetNextElement("battery");
+    }
+  }
+
   this->connections.push_back(event::Events::ConnectWorldUpdateBegin(
       boost::bind(&Link::Update, this, _1)));
 
@@ -255,45 +219,86 @@ void Link::Init()
     }
   }
 
+  // Initialize all the batteries
+  for (auto &battery : this->batteries)
+  {
+    battery->Init();
+  }
+
   this->initialized = true;
 }
 
 //////////////////////////////////////////////////
 void Link::Fini()
 {
+  this->attachedModels.clear();
   this->parentJoints.clear();
   this->childJoints.clear();
   this->collisions.clear();
   this->inertial.reset();
+  this->batteries.clear();
 
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-       iter != this->sensors.end(); ++iter)
+  // Remove all the sensors attached to the link
+  for (auto const &sensor : this->sensors)
   {
-    sensors::remove_sensor(*iter);
+    event::Events::removeSensor(sensor);
   }
+
   this->sensors.clear();
 
-  for (Visuals_M::iterator iter = this->visuals.begin();
-       iter != this->visuals.end(); ++iter)
+  // Clean up visuals
+  // FIXME: Do we really need to send 2 msgs to delete a visual?!
+  if (this->visPub && this->requestPub)
   {
-    msgs::Request *msg = msgs::CreateRequest("entity_delete",
-        boost::lexical_cast<std::string>(iter->second.id()));
-    this->requestPub->Publish(*msg, true);
-  }
+    for (auto iter : this->visuals)
+    {
+      auto deleteMsg = msgs::CreateRequest("entity_delete",
+          std::to_string(iter.second.id()));
+      this->requestPub->Publish(*deleteMsg, true);
 
-  for (std::vector<std::string>::iterator iter = this->cgVisuals.begin();
-       iter != this->cgVisuals.end(); ++iter)
-  {
-    msgs::Request *msg = msgs::CreateRequest("entity_delete", *iter);
-    this->requestPub->Publish(*msg, true);
+      msgs::Visual msg;
+      msg.set_name(iter.second.name());
+      msg.set_id(iter.second.id());
+      if (this->parent)
+      {
+        msg.set_parent_name(this->parent->GetScopedName());
+        msg.set_parent_id(this->parent->GetId());
+      }
+      else
+      {
+        msg.set_parent_name("");
+        msg.set_parent_id(0);
+      }
+      msg.set_delete_me(true);
+      this->visPub->Publish(msg);
+    }
   }
+  this->visuals.clear();
 
 #ifdef HAVE_OPENAL
-  this->world->GetPhysicsEngine()->GetContactManager()->RemoveFilter(
-      this->GetScopedName() + "/audio_collision");
+  if (this->world && this->world->GetPhysicsEngine() &&
+      this->world->GetPhysicsEngine()->GetContactManager())
+  {
+    this->world->GetPhysicsEngine()->GetContactManager()->RemoveFilter(
+        this->GetScopedName() + "/audio_collision");
+  }
+  this->audioContactsSub.reset();
   this->audioSink.reset();
+  this->audioSources.clear();
 #endif
 
+  // Clean transport
+  {
+    this->dataPub.reset();
+    this->visPub.reset();
+
+    this->wrenchSub.reset();
+  }
+  this->connections.clear();
+
+  delete this->publishDataMutex;
+  this->publishDataMutex = NULL;
+
   Entity::Fini();
 }
 
@@ -375,6 +380,21 @@ void Link::UpdateParameters(sdf::ElementPtr _sdf)
       collisionElem = collisionElem->GetNextElement("collision");
     }
   }
+
+  // Update the battery information
+  if (this->sdf->HasElement("battery"))
+  {
+    sdf::ElementPtr batteryElem = this->sdf->GetElement("battery");
+    while (batteryElem)
+    {
+      common::BatteryPtr battery = this->Battery(
+          batteryElem->Get<std::string>("name"));
+
+      if (battery)
+        battery->UpdateParameters(batteryElem);
+      batteryElem = batteryElem->GetNextElement("battery");
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -485,6 +505,12 @@ void Link::Update(const common::UpdateInfo & /*_info*/)
       this->ProcessWrenchMsg(it);
     }
   }
+
+  // Update the batteries.
+  for (auto &battery : this->batteries)
+  {
+    battery->Update();
+  }
 }
 
 /////////////////////////////////////////////////
@@ -791,6 +817,11 @@ void Link::FillMsg(msgs::Link &_msg)
   _msg.set_enabled(this->GetEnabled());
   msgs::Set(_msg.mutable_pose(), relPose.Ign());
 
+  // The visual msgs name might not have been set if the link was created
+  // dynamically without using SDF.
+  if (!this->visualMsg->has_name())
+    this->visualMsg->set_name(this->GetScopedName());
+
   msgs::Set(this->visualMsg->mutable_pose(), relPose.Ign());
   _msg.add_visual()->CopyFrom(*this->visualMsg);
 
@@ -813,12 +844,18 @@ void Link::FillMsg(msgs::Link &_msg)
     }
   }
 
-  for (std::vector<std::string>::iterator iter = this->sensors.begin();
-      iter != this->sensors.end(); ++iter)
+  // Add in the sensor data.
+  if (this->sdf->HasElement("sensor"))
   {
-    sensors::SensorPtr sensor = sensors::get_sensor(*iter);
-    if (sensor)
-      sensor->FillMsg(*_msg.add_sensor());
+    sdf::ElementPtr sensorElem = this->sdf->GetElement("sensor");
+    while (sensorElem)
+    {
+      msgs::Sensor *msg = _msg.add_sensor();
+      msg->CopyFrom(msgs::SensorFromSDF(sensorElem));
+      msg->set_parent(this->GetScopedName());
+      msg->set_parent_id(this->GetId());
+      sensorElem = sensorElem->GetNextElement("sensor");
+    }
   }
 
   if (this->visuals.empty())
@@ -849,6 +886,14 @@ void Link::FillMsg(msgs::Link &_msg)
 
   if (this->IsCanonicalLink())
     _msg.set_canonical(true);
+
+  // Fill message with battery information
+  for (auto &battery : this->batteries)
+  {
+    msgs::Battery *bat = _msg.add_battery();
+    bat->set_name(battery->Name());
+    bat->set_voltage(battery->Voltage());
+  }
 }
 
 //////////////////////////////////////////////////
@@ -877,7 +922,9 @@ void Link::ProcessMsg(const msgs::Link &_msg)
   {
     this->inertial->ProcessMsg(_msg.inertial());
     this->SetEnabled(true);
-    this->UpdateMass();
+    // Only update the Center of Mass if object is dynamic
+    if (!this->GetKinematic())
+      this->UpdateMass();
   }
 
   if (_msg.has_pose())
@@ -963,6 +1010,12 @@ void Link::OnPoseChange()
 void Link::SetState(const LinkState &_state)
 {
   this->SetWorldPose(_state.GetPose());
+  this->SetLinearVel(_state.GetVelocity().pos);
+  this->SetAngularVel(_state.GetVelocity().rot.GetAsEuler());
+  this->SetLinearAccel(_state.GetAcceleration().pos);
+  this->SetAngularAccel(_state.GetAcceleration().rot.GetAsEuler());
+  this->SetForce(_state.GetWrench().pos);
+  this->SetTorque(_state.GetWrench().rot.GetAsEuler());
 
   /*
   for (unsigned int i = 0; i < _state.GetCollisionStateCount(); ++i)
@@ -1002,6 +1055,11 @@ void Link::SetKinematic(const bool &/*_kinematic*/)
 /////////////////////////////////////////////////
 void Link::SetPublishData(bool _enable)
 {
+  // Skip if we're trying to disable after the publisher has already been
+  // cleared
+  if (!_enable && !this->dataPub)
+    return;
+
   {
     boost::recursive_mutex::scoped_lock lock(*this->publishDataMutex);
     if (this->publishData == _enable)
@@ -1020,6 +1078,7 @@ void Link::SetPublishData(bool _enable)
   else
   {
     this->dataPub.reset();
+    // Do we want to clear all of them though?
     this->connections.clear();
   }
 }
@@ -1039,6 +1098,120 @@ void Link::PublishData()
   }
 }
 
+//////////////////////////////////////////////////
+common::BatteryPtr Link::Battery(const std::string &_name) const
+{
+  common::BatteryPtr result;
+
+  for (auto &battery : this->batteries)
+  {
+    if (battery->Name() == _name)
+    {
+      result = battery;
+      break;
+    }
+  }
+
+  return result;
+}
+
+/////////////////////////////////////////////////
+common::BatteryPtr Link::Battery(const size_t _index) const
+{
+  if (_index < this->batteries.size())
+    return this->batteries[_index];
+  else
+    return common::BatteryPtr();
+}
+
+/////////////////////////////////////////////////
+size_t Link::BatteryCount() const
+{
+  return this->batteries.size();
+}
+
+//////////////////////////////////////////////////
+bool Link::VisualId(const std::string &_visName, uint32_t &_visualId) const
+{
+  for (auto &iter : this->visuals)
+  {
+    if (iter.second.name() == _visName ||
+        iter.second.name() == this->GetScopedName() + "::" + _visName)
+    {
+      _visualId = iter.first;
+      return true;
+    }
+  }
+  gzerr << "Trying to get unique ID of visual from invalid visual name["
+        << _visName << "] for link [" << this->GetScopedName() << "]\n";
+  return false;
+}
+
+//////////////////////////////////////////////////
+bool Link::VisualPose(const uint32_t _id, ignition::math::Pose3d &_pose) const
+{
+  auto iter = this->visuals.find(_id);
+  if (iter == this->visuals.end())
+  {
+    gzerr << "Trying to get pose of visual from invalid visual id[" << _id
+          << "] for link [" << this->GetScopedName() << "]\n";
+    return false;
+  }
+  const msgs::Visual &msg = iter->second;
+  if (msg.has_pose())
+  {
+    _pose = msgs::ConvertIgn(msg.pose());
+  }
+  else
+  {
+    // Pose wasn't specified on SDF, use default value
+    _pose = ignition::math::Pose3d::Zero;
+  }
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::SetVisualPose(const uint32_t _id,
+                         const ignition::math::Pose3d &_pose)
+{
+  auto iter = this->visuals.find(_id);
+  if (iter == this->visuals.end())
+  {
+    gzerr << "Trying to set pose of visual from invalid visual id[" << _id
+          << "] for link [" << this->GetScopedName() << "]\n";
+    return false;
+  }
+  msgs::Visual &msg = iter->second;
+  msgs::Set(msg.mutable_pose(), _pose);
+  std::string linkName = this->GetScopedName();
+  if (this->sdf->HasElement("visual"))
+  {
+    sdf::ElementPtr visualElem = this->sdf->GetElement("visual");
+    while (visualElem)
+    {
+      std::string visName = linkName + "::" +
+        visualElem->Get<std::string>("name");
+
+      // update visual msg if it exists
+      if (msg.name() == visName)
+      {
+        visualElem->GetElement("pose")->Set(_pose);
+        break;
+      }
+
+      visualElem = visualElem->GetNextElement("visual");
+    }
+  }
+  msgs::Visual visual;
+  visual.set_name(msg.name());
+  visual.set_id(_id);
+  visual.set_parent_name(linkName);
+  visual.set_parent_id(this->GetId());
+  msgs::Set(visual.mutable_pose(), _pose);
+  this->visPub->Publish(visual);
+  return true;
+}
+
 //////////////////////////////////////////////////
 void Link::OnCollision(ConstContactsPtr &_msg)
 {
@@ -1122,7 +1295,7 @@ void Link::SetScale(const math::Vector3 &_scale)
   // update the visual sdf to ensure cloning and saving has the correct values.
   this->UpdateVisualGeomSDF(_scale);
 
-  this->scale = _scale;
+  this->scale = _scale.Ign();
 }
 
 //////////////////////////////////////////////////
@@ -1148,8 +1321,8 @@ void Link::UpdateVisualGeomSDF(const math::Vector3 &_scale)
         // update radius the same way as collision shapes
         double radius = geomElem->GetElement("sphere")->Get<double>("radius");
         double newRadius = std::max(_scale.z, std::max(_scale.x, _scale.y));
-        double oldRadius = std::max(this->scale.z,
-            std::max(this->scale.x, this->scale.y));
+        double oldRadius = std::max(this->scale.Z(),
+            std::max(this->scale.X(), this->scale.Y()));
         geomElem->GetElement("sphere")->GetElement("radius")->Set(
             newRadius/oldRadius*radius);
       }
@@ -1158,13 +1331,13 @@ void Link::UpdateVisualGeomSDF(const math::Vector3 &_scale)
         // update radius the same way as collision shapes
         double radius = geomElem->GetElement("cylinder")->Get<double>("radius");
         double newRadius = std::max(_scale.x, _scale.y);
-        double oldRadius = std::max(this->scale.x, this->scale.y);
+        double oldRadius = std::max(this->scale.X(), this->scale.Y());
 
         double length = geomElem->GetElement("cylinder")->Get<double>("length");
         geomElem->GetElement("cylinder")->GetElement("radius")->Set(
             newRadius/oldRadius*radius);
         geomElem->GetElement("cylinder")->GetElement("length")->Set(
-            _scale.z/this->scale.z*length);
+            _scale.z/this->scale.Z()*length);
       }
       else if (geomElem->HasElement("mesh"))
         geomElem->GetElement("mesh")->GetElement("scale")->Set(_scale);
@@ -1437,3 +1610,11 @@ void Link::ProcessWrenchMsg(const msgs::Wrench &_msg)
   const ignition::math::Vector3d torque = msgs::ConvertIgn(_msg.torque());
   this->AddRelativeTorque(torque);
 }
+
+//////////////////////////////////////////////////
+void Link::LoadBattery(sdf::ElementPtr _sdf)
+{
+  common::BatteryPtr battery(new common::Battery());
+  battery->Load(_sdf);
+  this->batteries.push_back(battery);
+}
