@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License")
  * you may not use this file except in compliance with the License.
@@ -36,6 +36,7 @@ ModelState::ModelState(const ModelPtr _model, const common::Time &_realTime,
 : State(_model->GetName(), _realTime, _simTime, _iterations)
 {
   this->pose = _model->GetWorldPose();
+  this->scale = _model->Scale();
 
   // Copy all the links
   const Link_V links = _model->GetLinks();
@@ -45,6 +46,13 @@ ModelState::ModelState(const ModelPtr _model, const common::Time &_realTime,
           LinkState(*iter, _realTime, _simTime, _iterations)));
   }
 
+  // Copy all the models
+  for (const auto &m : _model->NestedModels())
+  {
+    this->modelStates.insert(std::make_pair(m->GetName(),
+          ModelState(m, _realTime, _simTime, _iterations)));
+  }
+
   // Copy all the joints
   /*const Joint_V joints = _model->GetJoints();
   for (Joint_V::const_iterator iter = joints.begin();
@@ -61,6 +69,7 @@ ModelState::ModelState(const ModelPtr _model)
         _model->GetWorld()->GetSimTime(), _model->GetWorld()->GetIterations())
 {
   this->pose = _model->GetWorldPose();
+  this->scale = _model->Scale();
 
   // Copy all the links
   const Link_V links = _model->GetLinks();
@@ -70,6 +79,13 @@ ModelState::ModelState(const ModelPtr _model)
           LinkState((*iter))));
   }
 
+  // Copy all the models
+  for (const auto &m : _model->NestedModels())
+  {
+    this->modelStates.insert(std::make_pair(m->GetName(),
+          ModelState(m)));
+  }
+
   // Copy all the joints
   /*const Joint_V joints = _model->GetJoints();
   for (Joint_V::const_iterator iter = joints.begin();
@@ -102,8 +118,10 @@ void ModelState::Load(const ModelPtr _model, const common::Time &_realTime,
   this->simTime = _simTime;
   this->iterations = _iterations;
   this->pose = _model->GetWorldPose();
+  this->scale = _model->Scale();
 
   // Load all the links
+  this->linkStates.clear();
   const Link_V links = _model->GetLinks();
   for (Link_V::const_iterator iter = links.begin(); iter != links.end(); ++iter)
   {
@@ -111,15 +129,11 @@ void ModelState::Load(const ModelPtr _model, const common::Time &_realTime,
         _iterations);
   }
 
-  // Remove links that no longer exist. We determine this by check the time
-  // stamp on each link.
-  for (LinkState_M::iterator iter = this->linkStates.begin();
-       iter != this->linkStates.end();)
+  // Load all the models
+  this->modelStates.clear();
+  for (const auto &m : _model->NestedModels())
   {
-    if (iter->second.GetRealTime() != this->realTime)
-      this->linkStates.erase(iter++);
-    else
-      ++iter;
+    this->modelStates[m->GetName()].Load(m, _realTime, _simTime, _iterations);
   }
 
   // Copy all the joints
@@ -155,6 +169,12 @@ void ModelState::Load(const sdf::ElementPtr _elem)
   else
     this->pose.Set(0, 0, 0, 0, 0, 0);
 
+  // Set the model scale
+  if (_elem->HasElement("scale"))
+    this->scale = _elem->Get<ignition::math::Vector3d>("scale");
+  else
+    this->scale.Set(1, 1, 1);
+
   // Set all the links
   this->linkStates.clear();
   if (_elem->HasElement("link"))
@@ -169,6 +189,20 @@ void ModelState::Load(const sdf::ElementPtr _elem)
     }
   }
 
+  // Set all the models
+  this->modelStates.clear();
+  if (_elem->HasElement("model"))
+  {
+    sdf::ElementPtr childElem = _elem->GetElement("model");
+
+    while (childElem)
+    {
+      this->modelStates.insert(std::make_pair(
+            childElem->Get<std::string>("name"), ModelState(childElem)));
+      childElem = childElem->GetNextElement("model");
+    }
+  }
+
   // Set all the joints
   /*this->jointStates.clear();
   if (_elem->HasElement("joint"))
@@ -190,6 +224,12 @@ const math::Pose &ModelState::GetPose() const
   return this->pose;
 }
 
+/////////////////////////////////////////////////
+const ignition::math::Vector3d &ModelState::Scale() const
+{
+  return this->scale;
+}
+
 /////////////////////////////////////////////////
 bool ModelState::IsZero() const
 {
@@ -201,13 +241,19 @@ bool ModelState::IsZero() const
     result = result && iter->second.IsZero();
   }
 
+  for (const auto &ms : this->modelStates)
+  {
+    result = result && ms.second.IsZero();
+  }
+
   /*for (JointState_M::const_iterator iter = this->jointStates.begin();
        iter != this->jointStates.end() && result; ++iter)
   {
     result = result && iter->second.IsZero();
   }*/
 
-  return result && this->pose == math::Pose::Zero;
+  return result && this->pose == math::Pose::Zero &&
+      this->scale == ignition::math::Vector3d::Zero;
 }
 
 /////////////////////////////////////////////////
@@ -256,19 +302,14 @@ LinkState ModelState::GetLinkState(const std::string &_linkName) const
   if (iter != this->linkStates.end())
     return iter->second;
 
-  gzthrow("Invalid link name[" + _linkName + "]");
+  gzerr << "Invalid link name[" + _linkName + "]" << std::endl;
   return LinkState();
 }
 
 /////////////////////////////////////////////////
 bool ModelState::HasLinkState(const std::string &_linkName) const
 {
-  // Search for the link name
-  LinkState_M::const_iterator iter = this->linkStates.find(_linkName);
-  if (iter != this->linkStates.end())
-    return true;
-
-  return false;
+  return this->linkStates.find(_linkName) != this->linkStates.end();
 }
 
 /////////////////////////////////////////////////
@@ -293,7 +334,7 @@ JointState ModelState::GetJointState(unsigned int _index) const
     return iter->second;
   }
 
-  gzthrow("Index is out of range");
+  gzerr << "Index is out of range" << std::endl;
   return JointState();
 }
 
@@ -304,7 +345,7 @@ JointState ModelState::GetJointState(const std::string &_jointName) const
   if (iter != this->jointStates.end())
     return iter->second;
 
-  gzthrow("Invalid joint name[" + _jointName + "]");
+  gzerr << "Invalid joint name[" + _jointName + "]" << std::endl;
   return JointState();
 }
 
@@ -325,6 +366,36 @@ const JointState_M &ModelState::GetJointStates() const
   return this->jointStates;
 }
 
+/////////////////////////////////////////////////
+unsigned int ModelState::NestedModelStateCount() const
+{
+  return this->modelStates.size();
+}
+
+/////////////////////////////////////////////////
+ModelState ModelState::NestedModelState(const std::string &_modelName) const
+{
+  // Search for the model name
+  auto iter = this->modelStates.find(_modelName);
+  if (iter != this->modelStates.end())
+    return iter->second;
+
+  gzerr << "Invalid model name[" + _modelName + "]" << std::endl;
+  return ModelState();
+}
+
+/////////////////////////////////////////////////
+bool ModelState::HasNestedModelState(const std::string &_modelName) const
+{
+  return this->modelStates.find(_modelName) != this->modelStates.end();
+}
+
+/////////////////////////////////////////////////
+const ModelState_M &ModelState::NestedModelStates() const
+{
+  return this->modelStates;
+}
+
 /////////////////////////////////////////////////
 ModelState &ModelState::operator=(const ModelState &_state)
 {
@@ -332,10 +403,12 @@ ModelState &ModelState::operator=(const ModelState &_state)
 
   // Copy the pose
   this->pose = _state.pose;
+  this->scale = _state.scale;
 
-  // Clear the link and joint states.
+  // Clear the link, joint, and model states.
   this->linkStates.clear();
   this->jointStates.clear();
+  this->modelStates.clear();
 
   // Copy the link states.
   for (LinkState_M::const_iterator iter =
@@ -344,6 +417,12 @@ ModelState &ModelState::operator=(const ModelState &_state)
     this->linkStates.insert(std::make_pair(iter->first, iter->second));
   }
 
+  // Copy the model states.
+  for (const auto &ms : _state.modelStates)
+  {
+    this->modelStates.insert(std::make_pair(ms.first, ms.second));
+  }
+
   // Copy the joint states.
   // for (JointState_M::const_iterator iter =
   //     _state.jointStates.begin(); iter != _state.jointStates.end(); ++iter)
@@ -362,6 +441,7 @@ ModelState ModelState::operator-(const ModelState &_state) const
   result.name = this->name;
   result.pose.pos = this->pose.pos - _state.pose.pos;
   result.pose.rot = _state.pose.rot.GetInverse() * this->pose.rot;
+  result.scale = this->scale - _state.scale;
 
   // Insert the link state diffs.
   for (LinkState_M::const_iterator iter =
@@ -384,6 +464,26 @@ ModelState ModelState::operator-(const ModelState &_state) const
     }
   }
 
+  // Insert the model state diffs.
+  for (const auto &ms : this->modelStates)
+  {
+    try
+    {
+      if (_state.HasNestedModelState(ms.second.GetName()))
+      {
+        ModelState state = ms.second - _state.NestedModelState(
+            ms.second.GetName());
+        if (!state.IsZero())
+          result.modelStates.insert(std::make_pair(state.GetName(), state));
+      }
+    }
+    catch(common::Exception &)
+    {
+      // Ignore exception, which is just the fact that a model state may not
+      // have been recorded.
+    }
+  }
+
   // Insert the joint state diffs.
   /*for (JointState_M::const_iterator iter =
        this->jointStates.begin(); iter != this->jointStates.end(); ++iter)
@@ -416,6 +516,7 @@ ModelState ModelState::operator+(const ModelState &_state) const
   result.name = this->name;
   result.pose.pos = this->pose.pos + _state.pose.pos;
   result.pose.rot = _state.pose.rot * this->pose.rot;
+  result.scale = this->scale + _state.scale;
 
   // Insert the link state diffs.
   for (LinkState_M::const_iterator iter =
@@ -437,6 +538,25 @@ ModelState ModelState::operator+(const ModelState &_state) const
     }
   }
 
+  // Insert the model state diffs.
+  for (const auto &ms : this->modelStates)
+  {
+    try
+    {
+      if (_state.HasNestedModelState(ms.second.GetName()))
+      {
+        ModelState state = ms.second + _state.NestedModelState(
+            ms.second.GetName());
+        result.modelStates.insert(std::make_pair(state.GetName(), state));
+      }
+    }
+    catch(common::Exception &)
+    {
+      // Ignore exception, which is just the fact that a model state may not
+      // have been recorded.
+    }
+  }
+
   // Insert the joint state diffs.
   for (JointState_M::const_iterator iter =
        this->jointStates.begin(); iter != this->jointStates.end(); ++iter)
@@ -467,6 +587,7 @@ void ModelState::FillSDF(sdf::ElementPtr _sdf)
 
   _sdf->GetAttribute("name")->Set(this->name);
   _sdf->GetElement("pose")->Set(this->pose);
+  _sdf->GetElement("scale")->Set(this->scale);
 
   for (LinkState_M::iterator iter = this->linkStates.begin();
        iter != this->linkStates.end(); ++iter)
@@ -475,6 +596,12 @@ void ModelState::FillSDF(sdf::ElementPtr _sdf)
     iter->second.FillSDF(elem);
   }
 
+  for (auto &ms : this->modelStates)
+  {
+    sdf::ElementPtr elem = _sdf->AddElement("model");
+    ms.second.FillSDF(elem);
+  }
+
   for (JointState_M::iterator iter = this->jointStates.begin();
        iter != this->jointStates.end(); ++iter)
   {
@@ -494,6 +621,11 @@ void ModelState::SetWallTime(const common::Time &_time)
     iter->second.SetWallTime(_time);
   }
 
+  for (auto &ms : this->modelStates)
+  {
+    ms.second.SetWallTime(_time);
+  }
+
   for (JointState_M::iterator iter = this->jointStates.begin();
        iter != this->jointStates.end(); ++iter)
   {
@@ -512,6 +644,11 @@ void ModelState::SetRealTime(const common::Time &_time)
     iter->second.SetRealTime(_time);
   }
 
+  for (auto &ms : this->modelStates)
+  {
+    ms.second.SetRealTime(_time);
+  }
+
   for (JointState_M::iterator iter = this->jointStates.begin();
        iter != this->jointStates.end(); ++iter)
   {
@@ -530,6 +667,11 @@ void ModelState::SetSimTime(const common::Time &_time)
     iter->second.SetSimTime(_time);
   }
 
+  for (auto &ms : this->modelStates)
+  {
+    ms.second.SetSimTime(_time);
+  }
+
   for (JointState_M::iterator iter = this->jointStates.begin();
        iter != this->jointStates.end(); ++iter)
   {
@@ -545,6 +687,11 @@ void ModelState::SetIterations(const uint64_t _iterations)
   for (auto &linkState : this->linkStates)
     linkState.second.SetIterations(_iterations);
 
+  for (auto &ms : this->modelStates)
+  {
+    ms.second.SetIterations(_iterations);
+  }
+
   for (auto &jointState : this->jointStates)
     jointState.second.SetIterations(_iterations);
 }
