@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -49,11 +49,25 @@ BulletJoint::BulletJoint(BasePtr _parent)
 //////////////////////////////////////////////////
 BulletJoint::~BulletJoint()
 {
-  delete this->constraint;
+  this->Fini();
+}
+
+//////////////////////////////////////////////////
+void BulletJoint::Fini()
+{
+  if (this->constraint && this->bulletWorld)
+  {
+    this->bulletWorld->removeConstraint(this->constraint);
+    delete this->constraint;
+  }
   this->constraint = NULL;
-  delete this->feedback;
-  this->feedback = NULL;
   this->bulletWorld = NULL;
+
+  if (this->feedback)
+    delete this->feedback;
+  this->feedback = NULL;
+
+  Joint::Fini();
 }
 
 //////////////////////////////////////////////////
@@ -143,6 +157,8 @@ bool BulletJoint::AreConnected(LinkPtr _one, LinkPtr _two) const
 //////////////////////////////////////////////////
 void BulletJoint::Detach()
 {
+  this->applyDamping.reset();
+
   this->childLink.reset();
   this->parentLink.reset();
   if (this->constraint && this->bulletWorld)
