@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,6 +45,7 @@ DARTLink::DARTLink(EntityPtr _parent)
 DARTLink::~DARTLink()
 {
   delete this->dataPtr;
+  this->dataPtr = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -235,12 +236,12 @@ void DARTLink::Init()
 
       SurfaceParamsPtr surface = collision->GetSurface();
       GZ_ASSERT(surface, "Surface pointer for is invalid");
-      FrictionPyramidPtr friction = surface->GetFrictionPyramid();
+      FrictionPyramidPtr friction = surface->FrictionPyramid();
       GZ_ASSERT(friction, "Friction pointer for is invalid");
 
       numCollisions++;
-      hackAvgMu1 += friction->GetMuPrimary();
-      hackAvgMu2 += friction->GetMuSecondary();
+      hackAvgMu1 += friction->MuPrimary();
+      hackAvgMu2 += friction->MuSecondary();
     }
   }
 
@@ -411,7 +412,7 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
   }
   else
   {
-    gzdbg << "DARTLink::SetLinearVel() doesn't make sense if the parent joint"
+    gzdbg << "DARTLink::SetLinearVel() doesn't make sense if the parent joint "
           << "is not free joint (6-dof).\n";
   }
 }
@@ -477,7 +478,7 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
   }
   else
   {
-    gzdbg << "DARTLink::SetLinearVel() doesn't make sense if the parent joint"
+    gzdbg << "DARTLink::SetLinearVel() doesn't make sense if the parent joint "
           << "is not free joint (6-dof).\n";
   }
 }
