@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -158,30 +158,8 @@ ODEPhysics::ODEPhysics(WorldPtr _world)
 //////////////////////////////////////////////////
 ODEPhysics::~ODEPhysics()
 {
-  dCloseODE();
+  this->Fini();
 
-  dJointGroupDestroy(this->dataPtr->contactGroup);
-
-  // Delete all the joint feedbacks.
-  for (std::vector<ODEJointFeedback*>::iterator iter =
-      this->dataPtr->jointFeedbacks.begin(); iter !=
-          this->dataPtr->jointFeedbacks.end(); ++iter)
-  {
-    delete *iter;
-  }
-  this->dataPtr->jointFeedbacks.clear();
-
-  if (this->dataPtr->spaceId)
-  {
-    dSpaceSetCleanup(this->dataPtr->spaceId, 0);
-    dSpaceDestroy(this->dataPtr->spaceId);
-  }
-
-  if (this->dataPtr->worldId)
-    dWorldDestroy(this->dataPtr->worldId);
-
-  this->dataPtr->spaceId = NULL;
-  this->dataPtr->worldId = NULL;
   delete this->dataPtr;
   this->dataPtr = NULL;
 }
@@ -230,12 +208,12 @@ void ODEPhysics::Load(sdf::ElementPtr _sdf)
   dWorldSetAutoDisableAngularThreshold(this->dataPtr->worldId, 0.1);
   dWorldSetAutoDisableSteps(this->dataPtr->worldId, 5);
 
-  math::Vector3 g = this->sdf->Get<math::Vector3>("gravity");
+  auto g = this->world->Gravity();
 
-  if (g == math::Vector3(0, 0, 0))
+  if (g == ignition::math::Vector3d::Zero)
     gzwarn << "Gravity vector is (0, 0, 0). Objects will float.\n";
 
-  dWorldSetGravity(this->dataPtr->worldId, g.x, g.y, g.z);
+  dWorldSetGravity(this->dataPtr->worldId, g.X(), g.Y(), g.Z());
 
   if (odeElem->HasElement("constraints"))
   {
@@ -470,6 +448,30 @@ void ODEPhysics::UpdatePhysics()
 //////////////////////////////////////////////////
 void ODEPhysics::Fini()
 {
+  dCloseODE();
+
+  dJointGroupDestroy(this->dataPtr->contactGroup);
+
+  // Delete all the joint feedbacks.
+  for (auto iter = this->dataPtr->jointFeedbacks.begin();
+      iter != this->dataPtr->jointFeedbacks.end(); ++iter)
+  {
+    delete *iter;
+  }
+  this->dataPtr->jointFeedbacks.clear();
+
+  if (this->dataPtr->spaceId)
+  {
+    dSpaceSetCleanup(this->dataPtr->spaceId, 0);
+    dSpaceDestroy(this->dataPtr->spaceId);
+  }
+
+  if (this->dataPtr->worldId)
+    dWorldDestroy(this->dataPtr->worldId);
+  this->dataPtr->worldId = NULL;
+
+  this->dataPtr->spaceId = NULL;
+
   PhysicsEngine::Fini();
 }
 
@@ -917,7 +919,7 @@ void ODEPhysics::SetStepType(const std::string &_type)
 //////////////////////////////////////////////////
 void ODEPhysics::SetGravity(const gazebo::math::Vector3 &_gravity)
 {
-  this->sdf->GetElement("gravity")->Set(_gravity);
+  this->world->SetGravitySDF(_gravity.Ign());
   dWorldSetGravity(this->dataPtr->worldId, _gravity.x, _gravity.y, _gravity.z);
 }
 
@@ -1071,6 +1073,7 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
                          dContactSoftERP |
                          dContactSoftCFM |
                          dContactApprox1 |
+                         dContactApprox3 |
                          dContactSlip1 |
                          dContactSlip2;
 
@@ -1093,7 +1096,7 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   //                                _collision2->surface->softCFM);
 
   // assign fdir1 if not set as 0
-  math::Vector3 fd = surf1->GetFrictionPyramid()->direction1;
+  math::Vector3 fd = surf1->FrictionPyramid()->direction1;
   if (fd != math::Vector3::Zero)
   {
     // fdir1 is in body local frame, rotate it into world frame
@@ -1107,10 +1110,10 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   /// As a hack, we'll simply compare mu1 from
   /// both surfaces for now, and use fdir1 specified by
   /// surface with smaller mu1.
-  math::Vector3 fd2 = surf2->GetFrictionPyramid()->direction1;
+  math::Vector3 fd2 = surf2->FrictionPyramid()->direction1;
   if (fd2 != math::Vector3::Zero && (fd == math::Vector3::Zero ||
-        surf1->GetFrictionPyramid()->GetMuPrimary() >
-        surf2->GetFrictionPyramid()->GetMuPrimary()))
+        surf1->FrictionPyramid()->MuPrimary() >
+        surf2->FrictionPyramid()->MuPrimary()))
   {
     // fdir1 is in body local frame, rotate it into world frame
     fd2 = _collision2->GetWorldPose().rot.RotateVector(fd2);
@@ -1133,17 +1136,86 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   }
 
   // Set the friction coefficients.
-  contact.surface.mu = std::min(surf1->GetFrictionPyramid()->GetMuPrimary(),
-                                surf2->GetFrictionPyramid()->GetMuPrimary());
-  contact.surface.mu2 = std::min(surf1->GetFrictionPyramid()->GetMuSecondary(),
-                                 surf2->GetFrictionPyramid()->GetMuSecondary());
-
+  contact.surface.mu = std::min(surf1->FrictionPyramid()->MuPrimary(),
+                                surf2->FrictionPyramid()->MuPrimary());
+  contact.surface.mu2 = std::min(surf1->FrictionPyramid()->MuSecondary(),
+                                 surf2->FrictionPyramid()->MuSecondary());
+  contact.surface.mu3 = std::min(surf1->FrictionPyramid()->MuTorsion(),
+                                 surf2->FrictionPyramid()->MuTorsion());
 
   // Set the slip values
   contact.surface.slip1 = std::min(surf1->slip1,
                                    surf2->slip1);
   contact.surface.slip2 = std::min(surf1->slip2,
                                    surf2->slip2);
+  contact.surface.slip3 = std::min(surf1->slipTorsion,
+                                   surf2->slipTorsion);
+
+  // Combine torsional friction patch radius values
+  contact.surface.patch_radius =
+      std::max(surf1->FrictionPyramid()->PatchRadius(),
+               surf2->FrictionPyramid()->PatchRadius());
+
+  // For torsional friction, the curvature is combined using
+  //   1/R = 1/R1 + 1/R2
+  // we can consider doing the same for the patch radius
+  double curv1 = 0;
+  if (surf1->FrictionPyramid()->SurfaceRadius() > 0)
+    curv1 = 1 / surf1->FrictionPyramid()->SurfaceRadius();
+
+  double curv2 = 0;
+  if (surf2->FrictionPyramid()->SurfaceRadius() > 0)
+    curv2 = 1 / surf2->FrictionPyramid()->SurfaceRadius();
+
+  double curvSum = curv1 + curv2;
+  contact.surface.surface_radius = 0;
+  if (curvSum > 0)
+    contact.surface.surface_radius = 1 / curvSum;
+
+  /// \todo Not sure how to combine these logic flags
+  /// If user wanted to use patch radius, but got settings
+  /// overwritten by the logic combination, how do we make sure the
+  /// the surface radius is specified or makes sense?
+  contact.surface.use_patch_radius =
+      surf1->FrictionPyramid()->UsePatchRadius() &&
+      surf2->FrictionPyramid()->UsePatchRadius();
+
+  if (contact.surface.mu3 > 0)
+  {
+    // Patch radius
+    if ((contact.surface.use_patch_radius &&
+        contact.surface.patch_radius > 0) ||
+    // Surface radius
+        (!contact.surface.use_patch_radius &&
+        contact.surface.surface_radius > 0))
+    {
+      contact.surface.mode |= dContactMu3;
+
+      if (contact.surface.slip3 > 0)
+      {
+        contact.surface.mode |= dContactSlip3;
+      }
+    }
+  }
+
+  // Set the elastic modulus
+  // Using Hertzian contact
+  // equation 5.26 from Contact Mechanics and Friction by Popov
+  double nu1 = surf1->FrictionPyramid()->PoissonsRatio();
+  double nu2 = surf2->FrictionPyramid()->PoissonsRatio();
+  double e1 = surf1->FrictionPyramid()->ElasticModulus();
+  double e2 = surf2->FrictionPyramid()->ElasticModulus();
+  if (e1 > 0 && e2 > 0)
+  {
+    contact.surface.elastic_modulus = 1.0 /
+      ((1.0 - nu1*nu1)/e1 + (1.0 - nu2*nu2)/e2);
+
+    // Turn on Contact Elastic Modulus model if elastic modulus > 0
+    if (contact.surface.elastic_modulus > 0.0)
+    {
+      contact.surface.mode |= dContactEM;
+    }
+  }
 
   // Set the bounce values
   contact.surface.bounce = std::min(surf1->bounce,
@@ -1410,6 +1482,11 @@ bool ODEPhysics::SetParam(const std::string &_key, const boost::any &_value)
       dWorldSetQuickStepContactResidualSmoothing(this->dataPtr->worldId,
         boost::any_cast<double>(_value));
     }
+    else if (_key == "contact_sor_scale")
+    {
+      dWorldSetQuickStepContactSORScalingFactor(this->dataPtr->worldId,
+        boost::any_cast<double>(_value));
+    }
     else if (_key == "thread_position_correction")
     {
       dWorldSetQuickStepThreadPositionCorrection(this->dataPtr->worldId,
@@ -1502,6 +1579,8 @@ bool ODEPhysics::GetParam(const std::string &_key, boost::any &_value) const
     _value = dWorldGetQuickStepInertiaRatioReduction(this->dataPtr->worldId);
   else if (_key == "contact_residual_smoothing")
     _value = dWorldGetQuickStepContactResidualSmoothing(this->dataPtr->worldId);
+  else if (_key == "contact_sor_scale")
+    _value = dWorldGetQuickStepContactSORScalingFactor(this->dataPtr->worldId);
   else if (_key == "thread_position_correction")
     _value = dWorldGetQuickStepThreadPositionCorrection(this->dataPtr->worldId);
   else if (_key == "experimental_row_reordering")
