@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -96,19 +96,6 @@ double SimbodyHingeJoint::GetVelocity(unsigned int _index) const
   }
 }
 
-//////////////////////////////////////////////////
-void SimbodyHingeJoint::SetMaxForce(unsigned int /*_index*/, double /*_t*/)
-{
-  gzdbg << "SetMaxForce: doesn't make sense in simbody...\n";
-}
-
-//////////////////////////////////////////////////
-double SimbodyHingeJoint::GetMaxForce(unsigned int /*_index*/)
-{
-  gzdbg << "GetMaxForce: doesn't make sense in simbody...\n";
-  return 0;
-}
-
 //////////////////////////////////////////////////
 void SimbodyHingeJoint::SetForceImpl(unsigned int _index, double _torque)
 {
