@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -64,7 +64,7 @@ void ImuSensor_TEST::BasicImuSensorCheck(const std::string &_physicsEngine)
   mgr->Update();
 
   // Get a pointer to the IMU sensor
-  sensors::ImuSensorPtr sensor = boost::dynamic_pointer_cast<sensors::ImuSensor>
+  sensors::ImuSensorPtr sensor = std::dynamic_pointer_cast<sensors::ImuSensor>
     (mgr->GetSensor(sensorName));
 
   // Make sure the above dynamic cast worked.
@@ -72,7 +72,7 @@ void ImuSensor_TEST::BasicImuSensorCheck(const std::string &_physicsEngine)
 
   EXPECT_EQ(sensor->AngularVelocity(), ignition::math::Vector3d::Zero);
   EXPECT_EQ(sensor->LinearAcceleration(), ignition::math::Vector3d::Zero);
-  EXPECT_EQ(sensor->Orientation(), ignition::math::Quaterniond(0, 0, 0, 0));
+  EXPECT_EQ(sensor->Orientation(), ignition::math::Quaterniond::Identity);
 }
 
 /////////////////////////////////////////////////
@@ -100,16 +100,16 @@ void ImuSensor_TEST::LinearAccelerationTest(const std::string &_physicsEngine)
 
   std::string modelName = "imuModel";
   std::string imuSensorName = "imuSensor";
-  math::Pose modelPose(0, 0, z, 0, 0, 0);
+  ignition::math::Pose3d modelPose(0, 0, z, 0, 0, 0);
 
   std::string topic = "~/" + imuSensorName + "_" + _physicsEngine;
   // spawn imu sensor
   SpawnUnitImuSensor(modelName, imuSensorName,
-      "box", topic, modelPose.pos, modelPose.rot.GetAsEuler());
+      "box", topic, modelPose.Pos(), modelPose.Rot().Euler());
 
   sensors::SensorPtr sensor = sensors::get_sensor(imuSensorName);
   sensors::ImuSensorPtr imuSensor =
-      boost::dynamic_pointer_cast<sensors::ImuSensor>(sensor);
+      std::dynamic_pointer_cast<sensors::ImuSensor>(sensor);
 
   ASSERT_TRUE(imuSensor != NULL);
 
@@ -118,7 +118,7 @@ void ImuSensor_TEST::LinearAccelerationTest(const std::string &_physicsEngine)
 
   EXPECT_EQ(imuSensor->AngularVelocity(), ignition::math::Vector3d::Zero);
   EXPECT_EQ(imuSensor->LinearAcceleration(), ignition::math::Vector3d::Zero);
-  EXPECT_EQ(imuSensor->Orientation(), ignition::math::Quaterniond(0, 0, 0, 0));
+  EXPECT_EQ(imuSensor->Orientation(), ignition::math::Quaterniond::Identity);
 
   // step world and verify imu's linear acceleration is zero on free fall
   world->Step(200);
