@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include <Winsock2.h>
 #endif
 
+#include <boost/algorithm/string.hpp>
 #include <stdio.h>
 #include <string>
 #include <cmath>
@@ -61,6 +62,13 @@ std::string gazebo::custom_exec(std::string _cmd)
   return result;
 }
 
+/////////////////////////////////////////////////
+void RenderingFixture::SetUp()
+{
+  // start rendering in test thread
+  rendering::load();
+}
+
 /////////////////////////////////////////////////
 ServerFixture::ServerFixture()
 {
@@ -149,16 +157,39 @@ void ServerFixture::Load(const std::string &_worldFilename, bool _paused)
 
 /////////////////////////////////////////////////
 void ServerFixture::Load(const std::string &_worldFilename,
-                  bool _paused, const std::string &_physics,
-                  const std::vector<std::string> &_systemPlugins)
+                         bool _paused, const std::string &_physics,
+                         const std::vector<std::string> &_systemPlugins)
+{
+  std::string params = _worldFilename;
+  if (!_physics.empty())
+    params += " -e " + _physics;
+  if (_paused)
+    params += " -u";
+  for (auto plugin : _systemPlugins)
+    params += " -s " + plugin;
+
+  this->LoadArgs(params);
+}
+
+/////////////////////////////////////////////////
+void ServerFixture::LoadArgs(const std::string &_args)
 {
   delete this->server;
   this->server = NULL;
 
+  // Split the string into a vector of parameters.
+  std::vector<std::string> params;
+  std::string args = _args;
+  boost::trim_if(args, boost::is_any_of("\t "));
+  boost::split(params, args, boost::is_any_of("\t "), boost::token_compress_on);
+
+  bool paused = false;
+  if (std::find(params.begin(), params.end(), "-u") != params.end())
+    paused = true;
+
   // Create, load, and run the server in its own thread
   this->serverThread = new boost::thread(
-     boost::bind(&ServerFixture::RunServer, this, _worldFilename,
-                 _paused, _physics, _systemPlugins));
+    boost::bind(&ServerFixture::RunServer, this, params));
 
   // Wait for the server to come up
   // Use a 60 second timeout.
@@ -167,14 +198,13 @@ void ServerFixture::Load(const std::string &_worldFilename,
          ++waitCount < maxWaitCount)
     common::Time::MSleep(100);
   gzdbg << "ServerFixture load in "
-         << static_cast<double>(waitCount)/10.0
-         << " seconds, timeout after "
-         << static_cast<double>(maxWaitCount)/10.0
-         << " seconds\n";
+        << static_cast<double>(waitCount)/10.0
+        << " seconds, timeout after "
+        << static_cast<double>(maxWaitCount)/10.0
+        << " seconds\n";
 
   if (waitCount >= maxWaitCount)
-    this->launchTimeoutFailure(
-        "while waiting for Load() function", waitCount);
+    this->launchTimeoutFailure("while waiting for Load() function", waitCount);
 
   this->node = transport::NodePtr(new transport::Node());
   ASSERT_NO_THROW(this->node->Init());
@@ -195,9 +225,12 @@ void ServerFixture::Load(const std::string &_worldFilename,
   waitCount = 0;
   maxWaitCount = 3000;
   while ((!physics::get_world() ||
-           physics::get_world()->IsPaused() != _paused) &&
+           physics::get_world()->IsPaused() != paused) &&
          ++waitCount < maxWaitCount)
+  {
     common::Time::MSleep(100);
+  }
+
   ASSERT_LT(waitCount, maxWaitCount);
 
   this->factoryPub->WaitForConnection();
@@ -205,46 +238,47 @@ void ServerFixture::Load(const std::string &_worldFilename,
 }
 
 /////////////////////////////////////////////////
-void ServerFixture::RunServer(const std::string &_worldFilename)
+void ServerFixture::RunServer(const std::vector<std::string> &_args)
 {
-  this->RunServer(_worldFilename, false, "");
-}
+  // Make room for an extra parameter (gzserver).
+  int argc = _args.size() + 1;
+  char **argv = new char* [argc];
+
+  // The first parameter is the name of the program.
+  const char *cmd = "gzserver";
+  argv[0] = strdup(cmd);
+
+  // Copy the command line parameters for gzserver.
+  for (size_t i = 0; i < _args.size(); ++i)
+    argv[i + 1] = strdup(_args.at(i).c_str());
 
-/////////////////////////////////////////////////
-void ServerFixture::RunServer(const std::string &_worldFilename, bool _paused,
-               const std::string &_physics,
-               const std::vector<std::string> &_systemPlugins)
-{
   ASSERT_NO_THROW(this->server = new Server());
 
-  for (auto const &plugin : _systemPlugins)
+  if (!this->server->ParseArgs(argc, argv))
   {
-    gazebo::addPlugin(plugin);
+    ASSERT_NO_THROW(delete this->server);
+    this->server = NULL;
+    return;
   }
 
-  this->server->PreLoad();
-
-  if (_physics.length())
-    ASSERT_NO_THROW(this->server->LoadFile(_worldFilename,
-                                           _physics));
-  else
-    ASSERT_NO_THROW(this->server->LoadFile(_worldFilename));
-
-  if (!rendering::get_scene(
-        gazebo::physics::get_world()->GetName()))
+  if (!rendering::get_scene(gazebo::physics::get_world()->GetName()))
   {
     ASSERT_NO_THROW(rendering::create_scene(
         gazebo::physics::get_world()->GetName(), false, true));
   }
 
-  ASSERT_NO_THROW(this->SetPause(_paused));
-
   ASSERT_NO_THROW(this->server->Run());
 
   ASSERT_NO_THROW(this->server->Fini());
 
   ASSERT_NO_THROW(delete this->server);
   this->server = NULL;
+
+  // Deallocate memory for the command line arguments allocated with strdup.
+  for (int i = 0; i < argc; ++i)
+    free(argv[i]);
+
+  delete[] argv;
 }
 
 /////////////////////////////////////////////////
@@ -473,10 +507,10 @@ void ServerFixture::GetFrame(const std::string &_cameraName,
   sensors::SensorPtr sensor = sensors::get_sensor(_cameraName);
   EXPECT_TRUE(sensor != NULL);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
 
-  _width = camSensor->GetImageWidth();
-  _height = camSensor->GetImageHeight();
+  _width = camSensor->ImageWidth();
+  _height = camSensor->ImageHeight();
 
   if (*_imgData)
   {
@@ -488,14 +522,14 @@ void ServerFixture::GetFrame(const std::string &_cameraName,
 
   this->gotImage = 0;
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
+    camSensor->Camera()->ConnectNewImageFrame(
         boost::bind(&ServerFixture::OnNewFrame,
                     this, _1, _2, _3, _4, _5));
 
   while (this->gotImage < 20)
     common::Time::MSleep(100);
 
-  camSensor->GetCamera()->DisconnectNewImageFrame(c);
+  camSensor->Camera()->DisconnectNewImageFrame(c);
 }
 
 /////////////////////////////////////////////////
@@ -663,6 +697,45 @@ void ServerFixture::SpawnRaySensor(const std::string &_modelName,
   WaitUntilSensorSpawn(_raySensorName, 100, 100);
 }
 
+/////////////////////////////////////////////////
+sensors::SonarSensorPtr ServerFixture::SpawnSonar(const std::string &_modelName,
+    const std::string &_sonarName,
+    const ignition::math::Pose3d &_pose,
+    const double _minRange,
+    const double _maxRange,
+    const double _radius)
+{
+  msgs::Factory msg;
+  std::ostringstream newModelStr;
+
+  newModelStr << "<sdf version='" << SDF_VERSION << "'>"
+    << "<model name ='" << _modelName << "'>"
+    << "<static>true</static>"
+    << "<pose>" << _pose << "</pose>"
+    << "<link name ='body'>"
+    << "  <sensor name ='" << _sonarName << "' type ='sonar'>"
+    << "    <sonar>"
+    << "      <min>" << _minRange << "</min>"
+    << "      <max>" << _maxRange << "</max>"
+    << "      <radius>" << _radius << "</radius>"
+    << "    </sonar>"
+    << "    <visualize>true</visualize>"
+    << "    <always_on>true</always_on>"
+    << "  </sensor>"
+    << "</link>"
+    << "</model>"
+    << "</sdf>";
+
+  msg.set_sdf(newModelStr.str());
+  this->factoryPub->Publish(msg);
+
+  WaitUntilEntitySpawn(_modelName, 100, 100);
+  WaitUntilSensorSpawn(_sonarName, 100, 100);
+  return std::dynamic_pointer_cast<sensors::SonarSensor>(
+      sensors::get_sensor(_sonarName));
+}
+
+
 /////////////////////////////////////////////////
 void ServerFixture::SpawnGpuRaySensor(const std::string &_modelName,
     const std::string &_raySensorName,
@@ -763,29 +836,53 @@ void ServerFixture::SpawnImuSensor(const std::string &_modelName,
 
   if (_noiseType.size() > 0)
   {
-    newModelStr << "      <noise>" << std::endl
-    << "        <type>" << _noiseType << "</type>" << std::endl
-    << "        <rate>" << std::endl
-    << "          <mean>" << _rateNoiseMean
-    << "</mean>" << std::endl
-    << "          <stddev>" << _rateNoiseStdDev
-    << "</stddev>" << std::endl
-    << "          <bias_mean>" << _rateBiasMean
-    << "</bias_mean>" << std::endl
-    << "          <bias_stddev>" << _rateBiasStdDev
-    << "</bias_stddev>" << std::endl
-    << "        </rate>" << std::endl
-    << "        <accel>" << std::endl
-    << "          <mean>" << _accelNoiseMean << "</mean>"
-    << std::endl
-    << "          <stddev>" << _accelNoiseStdDev << "</stddev>"
-    << std::endl
-    << "          <bias_mean>" << _accelBiasMean
-    << "</bias_mean>" << std::endl
-    << "          <bias_stddev>" << _accelBiasStdDev
-    << "</bias_stddev>" << std::endl
-    << "        </accel>" << std::endl
-    << "      </noise>" << std::endl;
+    newModelStr
+      << "<angular_velocity>\n"
+      << "<x><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _rateNoiseMean << "</mean>\n"
+      << "<stddev>" << _rateNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _rateBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _rateBiasStdDev << "</bias_stddev>\n"
+      << "</noise></x>\n"
+
+      << "<y><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _rateNoiseMean << "</mean>\n"
+      << "<stddev>" << _rateNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _rateBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _rateBiasStdDev << "</bias_stddev>\n"
+      << "</noise></y>\n"
+
+      << "<z><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _rateNoiseMean << "</mean>\n"
+      << "<stddev>" << _rateNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _rateBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _rateBiasStdDev << "</bias_stddev>\n"
+      << "</noise></z>\n"
+      << "</angular_velocity>\n"
+
+
+      << "<linear_acceleration>\n"
+      << "<x><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _accelNoiseMean << "</mean>\n"
+      << "<stddev>" << _accelNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _accelBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _accelBiasStdDev << "</bias_stddev>\n"
+      << "</noise></x>\n"
+
+      << "<y><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _accelNoiseMean << "</mean>\n"
+      << "<stddev>" << _accelNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _accelBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _accelBiasStdDev << "</bias_stddev>\n"
+      << "</noise></y>\n"
+
+      << "<z><noise type='" << _noiseType << "'>\n"
+      << "<mean>" << _accelNoiseMean << "</mean>\n"
+      << "<stddev>" << _accelNoiseStdDev << "</stddev>\n"
+      << "<bias_mean>" << _accelBiasMean << "</bias_mean>\n"
+      << "<bias_stddev>" << _accelBiasStdDev << "</bias_stddev>\n"
+      << "</noise></z>\n"
+      << "</linear_acceleration>\n";
   }
 
   newModelStr << "    </imu>" << std::endl
@@ -1156,6 +1253,32 @@ void ServerFixture::WaitUntilSensorSpawn(const std::string &_name,
            << ") exceeded while awaiting the spawn of " << _name;
 }
 
+/////////////////////////////////////////////////
+void ServerFixture::WaitUntilIteration(const uint32_t _goalIteration,
+    const int _sleepEach, const int _retries) const
+{
+  int i = 0;
+  auto world = physics::get_world();
+  while ((world->GetIterations() != _goalIteration) && (i < _retries))
+  {
+    ++i;
+    common::Time::MSleep(_sleepEach);
+  }
+}
+
+/////////////////////////////////////////////////
+void ServerFixture::WaitUntilSimTime(const common::Time &_goalTime,
+    const int _sleepEach, const int _retries) const
+{
+  int i = 0;
+  auto world = physics::get_world();
+  while ((world->GetSimTime() != _goalTime) && (i < _retries))
+  {
+    ++i;
+    common::Time::MSleep(_sleepEach);
+  }
+}
+
 /////////////////////////////////////////////////
 void ServerFixture::SpawnLight(const std::string &_name,
     const std::string &_type,
@@ -1231,8 +1354,8 @@ void ServerFixture::SpawnCylinder(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(),
-      ignition::math::Pose3d(_pos.Ign(), _rpy.Ign()));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
+      ignition::math::Quaterniond(_rpy.Ign())));
   msgs::AddCylinderLink(model, 1.0, 0.5, 1.0);
   auto link = model.mutable_link(0);
   link->set_name("body");
@@ -1296,8 +1419,8 @@ void ServerFixture::SpawnSphere(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(),
-      ignition::math::Pose3d(_pos.Ign(), _rpy.Ign()));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
+      ignition::math::Quaterniond(_rpy.Ign())));
   msgs::AddSphereLink(model, 1.0, _radius);
   auto link = model.mutable_link(0);
   link->set_name("body");
@@ -1328,8 +1451,8 @@ void ServerFixture::SpawnBox(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(),
-      ignition::math::Pose3d(_pos.Ign(), _rpy.Ign()));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
+      ignition::math::Quaterniond(_rpy.Ign())));
   msgs::AddBoxLink(model, 1.0, _size.Ign());
   auto link = model.mutable_link(0);
   link->set_name("body");
@@ -1396,8 +1519,8 @@ void ServerFixture::SpawnEmptyLink(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(),
-      ignition::math::Pose3d(_pos.Ign(), _rpy.Ign()));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
+      ignition::math::Quaterniond(_rpy.Ign())));
   model.add_link();
   model.mutable_link(0)->set_name("body");
 
