@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -106,6 +106,13 @@ void Publication::SetPrevMsg(uint32_t _pubId, MessagePtr _msg)
   this->prevMsgs[_pubId] = _msg;
 }
 
+//////////////////////////////////////////////////
+void Publication::ClearPrevMsgs()
+{
+  boost::mutex::scoped_lock lock(this->callbackMutex);
+  this->prevMsgs.clear();
+}
+
 //////////////////////////////////////////////////
 void Publication::AddTransport(const PublicationTransportPtr &_publink)
 {
@@ -359,6 +366,13 @@ unsigned int Publication::GetCallbackCount() const
   return this->callbacks.size();
 }
 
+//////////////////////////////////////////////////
+unsigned int Publication::PublisherCount() const
+{
+  boost::mutex::scoped_lock lock(this->callbackMutex);
+  return this->publishers.size();
+}
+
 //////////////////////////////////////////////////
 unsigned int Publication::GetNodeCount() const
 {
@@ -404,16 +418,28 @@ void Publication::AddPublisher(PublisherPtr _pub)
 //////////////////////////////////////////////////
 void Publication::RemovePublisher(PublisherPtr _pub)
 {
-  boost::mutex::scoped_lock lock(this->callbackMutex);
-
   GZ_ASSERT(_pub, "Received a NULL PublisherPtr");
 
-  // Find the publiser
-  std::vector<PublisherPtr>::iterator iter = std::find(
-      this->publishers.begin(), this->publishers.end(), _pub);
+  if (_pub)
+    this->RemovePublisher(_pub->Id());
+}
 
-  if (iter != this->publishers.end())
-    this->publishers.erase(iter);
+//////////////////////////////////////////////////
+bool Publication::RemovePublisher(const uint32_t _id)
+{
+  boost::mutex::scoped_lock lock(this->callbackMutex);
+
+  // Find and erase the publiser
+  for (auto pubIter = this->publishers.begin();
+       pubIter != this->publishers.end(); ++pubIter)
+  {
+    if ((*pubIter)->Id() == _id)
+    {
+      this->publishers.erase(pubIter);
+      return true;
+    }
+  }
+  return false;
 }
 
 //////////////////////////////////////////////////
@@ -489,8 +515,10 @@ void Publication::RemoveNodes()
 //////////////////////////////////////////////////
 MessagePtr Publication::GetPrevMsg(uint32_t _pubId)
 {
+  boost::mutex::scoped_lock lock(this->callbackMutex);
   if (this->prevMsgs.find(_pubId) != this->prevMsgs.end())
     return this->prevMsgs[_pubId];
   else
     return MessagePtr();
 }
+
