@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,8 @@
  * limitations under the License.
  *
 */
+#include <mutex>
+#include <functional>
 
 #include <ignition/math/Rand.hh>
 
@@ -32,7 +34,7 @@ class CameraSensor : public ServerFixture
 {
 };
 
-boost::mutex mutex;
+std::mutex mutex;
 
 unsigned char* img = NULL;
 unsigned char* img2 = NULL;
@@ -47,12 +49,176 @@ void OnNewCameraFrame(int* _imageCounter, unsigned char* _imageDest,
                   unsigned int _depth,
                   const std::string &_format)
 {
-  boost::mutex::scoped_lock lock(mutex);
+  std::lock_guard<std::mutex> lock(mutex);
   pixelFormat = _format;
   memcpy(_imageDest, _image, _width * _height * _depth);
   *_imageCounter += 1;
 }
 
+
+/////////////////////////////////////////////////
+TEST_F(CameraSensor, WorldReset)
+{
+  Load("worlds/empty_test.world");
+
+  // Make sure the render engine is available.
+  if (rendering::RenderEngine::Instance()->GetRenderPathType() ==
+      rendering::RenderEngine::NONE)
+  {
+    gzerr << "No rendering engine, unable to run camera test\n";
+    return;
+  }
+
+  // spawn sensors of various sizes to test speed
+  std::string modelName = "camera_model";
+  std::string cameraName = "camera_sensor";
+  unsigned int width  = 320;
+  unsigned int height = 240;
+  double updateRate = 10;
+  math::Pose setPose, testPose(
+      math::Vector3(-5, 0, 5), math::Quaternion(0, GZ_DTOR(15), 0));
+  SpawnCamera(modelName, cameraName, setPose.pos,
+      setPose.rot.GetAsEuler(), width, height, updateRate);
+  sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
+  sensors::CameraSensorPtr camSensor =
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+  imageCount = 0;
+  img = new unsigned char[width * height*3];
+  event::ConnectionPtr c =
+      camSensor->Camera()->ConnectNewImageFrame(
+      std::bind(&::OnNewCameraFrame, &imageCount, img,
+      std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+      std::placeholders::_4, std::placeholders::_5));
+  common::Timer timer;
+  timer.Start();
+
+  // let the camera render for 2 seconds at 10 Hz
+  int total_images = 20;
+  while (imageCount < total_images && timer.GetElapsed().Double() < 4)
+    common::Time::MSleep(10);
+  EXPECT_GE(imageCount, total_images);
+  common::Time dt = timer.GetElapsed();
+  EXPECT_GT(dt.Double(), 1.0);
+  EXPECT_LT(dt.Double(), 3.0);
+
+  // reset the world and verify
+  physics::WorldPtr world = physics::get_world("default");
+  ASSERT_TRUE(world != NULL);
+  common::Time simTime = world->GetSimTime().Double();
+  world->Reset();
+  EXPECT_TRUE(world->GetSimTime() == common::Time(0.0) ||
+      world->GetSimTime() < simTime);
+
+  // verify that the camera can continue to render and generate images at
+  // the specified rate
+  imageCount = 0;
+  timer.Reset();
+  timer.Start();
+  while (imageCount < total_images && timer.GetElapsed().Double() < 4)
+    common::Time::MSleep(10);
+  dt = timer.GetElapsed();
+  EXPECT_GE(imageCount, total_images);
+  EXPECT_GT(dt.Double(), 1.0);
+  EXPECT_LT(dt.Double(), 3.0);
+
+  camSensor->Camera()->DisconnectNewImageFrame(c);
+  delete [] img;
+}
+
+/////////////////////////////////////////////////
+TEST_F(CameraSensor, MultipleCameraSameName)
+{
+  Load("worlds/empty_test.world");
+
+  // Make sure the render engine is available.
+  if (rendering::RenderEngine::Instance()->GetRenderPathType() ==
+      rendering::RenderEngine::NONE)
+  {
+    gzerr << "No rendering engine, unable to run camera test\n";
+    return;
+  }
+
+  // spawn first camera sensor
+  std::string modelName = "camera_model";
+  std::string cameraName = "camera_sensor";
+  unsigned int width  = 320;
+  unsigned int height = 240;  // 106 fps
+  double updateRate = 10;
+  ignition::math::Pose3d setPose, testPose(
+      ignition::math::Vector3d(-5, 0, 5),
+      ignition::math::Quaterniond(0, GZ_DTOR(15), 0));
+  SpawnCamera(modelName, cameraName, setPose.Pos(),
+      setPose.Rot().Euler(), width, height, updateRate);
+  std::string sensorScopedName =
+      "default::" + modelName + "::body::" + cameraName;
+  sensors::SensorPtr sensor = sensors::get_sensor(sensorScopedName);
+  EXPECT_TRUE(sensor != NULL);
+  sensors::CameraSensorPtr camSensor =
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+  EXPECT_TRUE(camSensor != NULL);
+  rendering::CameraPtr camera = camSensor->Camera();
+  EXPECT_TRUE(camera != NULL);
+
+  // spawn second camera sensor with same name but attached to a different model
+  std::string modelName2 = modelName + "_2";
+  SpawnCamera(modelName2, cameraName, setPose.Pos(),
+      setPose.Rot().Euler(), width, height, updateRate);
+  std::string sensorScopedName2 =
+      "default::" + modelName2 + "::body::" + cameraName;
+  sensors::SensorPtr sensor2 = sensors::get_sensor(sensorScopedName2);
+  EXPECT_TRUE(sensor2 != NULL);
+  sensors::CameraSensorPtr camSensor2 =
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor2);
+  EXPECT_TRUE(camSensor2 != NULL);
+  rendering::CameraPtr camera2 = camSensor2->Camera();
+  EXPECT_TRUE(camera2 != NULL);
+
+  // verify that the sensors and cameras are not the same
+  EXPECT_TRUE(camSensor != camSensor2);
+  EXPECT_TRUE(camera != camera2);
+
+  // get camera scene and verify camera count
+  rendering::ScenePtr scene = camera->GetScene();
+  EXPECT_TRUE(scene != NULL);
+  EXPECT_EQ(scene->CameraCount(), 2u);
+
+  // remove the second camera sensor first and check that it does not remove
+  // the first one with the same name
+  sensors::remove_sensor(sensorScopedName2);
+  int sleep = 0;
+  int maxSleep = 10;
+  while (sensors::get_sensor(sensorScopedName2) != NULL && sleep < maxSleep)
+  {
+    common::Time::MSleep(100);
+    sleep++;
+  }
+  sensor2 = sensors::get_sensor(sensorScopedName2);
+  EXPECT_TRUE(sensor2 == NULL);
+  sensor = sensors::get_sensor(sensorScopedName);
+  EXPECT_TRUE(sensor != NULL);
+
+  // verify the first camera is still there
+  EXPECT_EQ(scene->CameraCount(), 1u);
+  EXPECT_TRUE(camera == scene->GetCamera(0));
+
+  std::string renderingCameraName = camera->Name();
+
+  // remove the first camera sensor and there should be no sensors or cameras
+  // left
+  sensors::remove_sensor(sensorScopedName);
+  sleep = 0;
+  while (sensors::get_sensor(sensorScopedName) != NULL && sleep < maxSleep)
+  {
+    common::Time::MSleep(100);
+    sleep++;
+  }
+  sensor = sensors::get_sensor(sensorScopedName);
+  EXPECT_TRUE(sensor == NULL);
+  camera = scene->GetCamera(renderingCameraName);
+  EXPECT_TRUE(camera == NULL);
+  EXPECT_EQ(scene->CameraCount(), 0u);
+}
+
 /////////////////////////////////////////////////
 TEST_F(CameraSensor, CheckThrottle)
 {
@@ -78,13 +244,13 @@ TEST_F(CameraSensor, CheckThrottle)
       setPose.rot.GetAsEuler(), width, height, updateRate);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   imageCount = 0;
   img = new unsigned char[width * height*3];
-  event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
-          _1, _2, _3, _4, _5));
+  event::ConnectionPtr c = camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
   common::Timer timer;
   timer.Start();
 
@@ -98,8 +264,8 @@ TEST_F(CameraSensor, CheckThrottle)
   gzdbg << "timer [" << dt.Double() << "] seconds rate [" << rate << "] fps\n";
   EXPECT_GT(rate, 7.0);
   EXPECT_LT(rate, 11.0);
-  camSensor->GetCamera()->DisconnectNewImageFrame(c);
-  delete img;
+  camSensor->Camera()->DisconnectNewImageFrame(c);
+  delete [] img;
 }
 
 /////////////////////////////////////////////////
@@ -129,14 +295,14 @@ TEST_F(CameraSensor, FillMsg)
       setPose.rot.GetAsEuler(), width, height, updateRate);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
 
   msgs::Sensor msg;
   sensor->FillMsg(msg);
 
   // Required fields
   EXPECT_EQ(msg.name(), cameraName);
-  EXPECT_EQ(msg.parent(), sensor->GetParentName());
+  EXPECT_EQ(msg.parent(), sensor->ParentName());
   EXPECT_EQ(msg.type(), "camera");
 
   // Optional fields
@@ -147,25 +313,25 @@ TEST_F(CameraSensor, FillMsg)
   EXPECT_EQ(msgs::ConvertIgn(msg.pose()), sensor->Pose());
 
   ASSERT_TRUE(msg.has_topic());
-  EXPECT_EQ(msg.topic(), sensor->GetTopic());
+  EXPECT_EQ(msg.topic(), sensor->Topic());
 
   ASSERT_TRUE(msg.has_update_rate());
-  EXPECT_EQ(msg.update_rate(), sensor->GetUpdateRate());
+  EXPECT_EQ(msg.update_rate(), sensor->UpdateRate());
 
   ASSERT_TRUE(msg.has_visualize());
-  EXPECT_EQ(msg.visualize(), sensor->GetVisualize());
+  EXPECT_EQ(msg.visualize(), sensor->Visualize());
 
   ASSERT_FALSE(msg.has_contact());
   ASSERT_FALSE(msg.has_ray());
   ASSERT_TRUE(msg.has_camera());
   auto cameraMsg = msg.camera();
-  auto cam = camSensor->GetCamera();
-  EXPECT_EQ(cameraMsg.horizontal_fov(), cam->GetHFOV().Radian());
-  EXPECT_EQ(cameraMsg.image_size().x(), camSensor->GetImageWidth());
-  EXPECT_EQ(cameraMsg.image_size().y(), camSensor->GetImageHeight());
-  EXPECT_EQ(cameraMsg.image_format(), cam->GetImageFormat());
-  EXPECT_EQ(cameraMsg.near_clip(), cam->GetNearClip());
-  EXPECT_EQ(cameraMsg.far_clip(), cam->GetFarClip());
+  auto cam = camSensor->Camera();
+  EXPECT_EQ(cameraMsg.horizontal_fov(), cam->HFOV().Radian());
+  EXPECT_EQ(cameraMsg.image_size().x(), camSensor->ImageWidth());
+  EXPECT_EQ(cameraMsg.image_size().y(), camSensor->ImageHeight());
+  EXPECT_EQ(cameraMsg.image_format(), cam->ImageFormat());
+  EXPECT_EQ(cameraMsg.near_clip(), cam->NearClip());
+  EXPECT_EQ(cameraMsg.far_clip(), cam->FarClip());
 }
 
 /////////////////////////////////////////////////
@@ -195,13 +361,14 @@ TEST_F(CameraSensor, UnlimitedTest)
       setPose.rot.GetAsEuler(), width, height, updateRate);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   imageCount = 0;
   img = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
-          _1, _2, _3, _4, _5));
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
   common::Timer timer;
   timer.Start();
   // time how long it takes to get N images
@@ -211,10 +378,10 @@ TEST_F(CameraSensor, UnlimitedTest)
   common::Time dt = timer.GetElapsed();
   double rate = static_cast<double>(total_images)/dt.Double();
   gzdbg << "timer [" << dt.Double() << "] seconds rate [" << rate << "] fps\n";
-  camSensor->GetCamera()->DisconnectNewImageFrame(c);
+  camSensor->Camera()->DisconnectNewImageFrame(c);
   EXPECT_GT(rate, 30.0);
 
-  delete img;
+  delete [] img;
 }
 
 /////////////////////////////////////////////////
@@ -248,12 +415,12 @@ TEST_F(CameraSensor, MultiSenseHigh)
       setPose.rot.GetAsEuler(), width, height, updateRate);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   imageCount = 0;
   img = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
           _1, _2, _3, _4, _5));
   common::Timer timer;
   timer.Start();
@@ -264,7 +431,7 @@ TEST_F(CameraSensor, MultiSenseHigh)
   common::Time dt = timer.GetElapsed();
   double rate = static_cast<double>(total_images)/dt.Double();
   gzdbg << "timer [" << dt.Double() << "] seconds rate [" << rate << "] fps\n";
-  camSensor->GetCamera()->DisconnectNewImageFrame(c);
+  camSensor->Camera()->DisconnectNewImageFrame(c);
   EXPECT_GT(rate, 24.0);
   EXPECT_LT(rate, 25.0);
 
@@ -303,12 +470,12 @@ TEST_F(CameraSensor, MultiSenseLow)
       setPose.rot.GetAsEuler(), width, height, updateRate);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   imageCount = 0;
   img = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
           _1, _2, _3, _4, _5));
   common::Timer timer;
   timer.Start();
@@ -319,7 +486,7 @@ TEST_F(CameraSensor, MultiSenseLow)
   common::Time dt = timer.GetElapsed();
   double rate = static_cast<double>(total_images)/dt.Double();
   gzdbg << "timer [" << dt.Double() << "] seconds rate [" << rate << "] fps\n";
-  camSensor->GetCamera()->DisconnectNewImageFrame(c);
+  camSensor->Camera()->DisconnectNewImageFrame(c);
   EXPECT_GT(rate, 24.0);
   EXPECT_LT(rate, 25.0);
 
@@ -360,23 +527,25 @@ TEST_F(CameraSensor, CheckNoise)
       "gaussian", noiseMean, noiseStdDev);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   sensor = sensors::get_sensor(cameraNameNoisy);
   sensors::CameraSensorPtr camSensorNoisy =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
 
   imageCount = 0;
   imageCount2 = 0;
   img = new unsigned char[width * height*3];
   img2 = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
-          _1, _2, _3, _4, _5));
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
   event::ConnectionPtr c2 =
-    camSensorNoisy->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount2, img2,
-          _1, _2, _3, _4, _5));
+    camSensorNoisy->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount2, img2,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
 
   // Get some images
   while (imageCount < 10 || imageCount2 < 10)
@@ -428,23 +597,25 @@ TEST_F(CameraSensor, CheckDistortion)
       "", 0, 0, true, -0.25349, 0.11868, 0.0, -0.00028, 0.00005, 0.5, 0.5);
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   sensor = sensors::get_sensor(cameraNameDistorted);
   sensors::CameraSensorPtr camSensorDistorted =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
 
   imageCount = 0;
   imageCount2 = 0;
   img = new unsigned char[width * height*3];
   img2 = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
-          _1, _2, _3, _4, _5));
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
   event::ConnectionPtr c2 =
-    camSensorDistorted->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount2, img2,
-          _1, _2, _3, _4, _5));
+    camSensorDistorted->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount2, img2,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
 
   // Get some images
   while (imageCount < 10 || imageCount2 < 10)
@@ -534,10 +705,10 @@ TEST_F(CameraSensor, CompareSideBySideCamera)
 
   sensors::SensorPtr sensor = sensors::get_sensor(cameraName);
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   sensor = sensors::get_sensor(cameraName2);
   sensors::CameraSensorPtr camSensor2 =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
 
   imageCount = 0;
   imageCount2 = 0;
@@ -546,13 +717,15 @@ TEST_F(CameraSensor, CompareSideBySideCamera)
   img2 = new unsigned char[width * height*3];
   unsigned char *prevImg2 = new unsigned char[width * height*3];
   event::ConnectionPtr c =
-    camSensor->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount, img,
-          _1, _2, _3, _4, _5));
+    camSensor->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount, img,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
   event::ConnectionPtr c2 =
-    camSensor2->GetCamera()->ConnectNewImageFrame(
-        boost::bind(&::OnNewCameraFrame, &imageCount2, img2,
-          _1, _2, _3, _4, _5));
+    camSensor2->Camera()->ConnectNewImageFrame(
+        std::bind(&::OnNewCameraFrame, &imageCount2, img2,
+          std::placeholders::_1, std::placeholders::_2, std::placeholders::_3,
+          std::placeholders::_4, std::placeholders::_5));
 
   while (imageCount < 10 || imageCount2 < 10)
     common::Time::MSleep(10);
@@ -580,7 +753,7 @@ TEST_F(CameraSensor, CompareSideBySideCamera)
       unsigned int diffMax2 = 0;
       double diffAvg2 = 0.0;
 
-      boost::mutex::scoped_lock lock(mutex);
+      std::lock_guard<std::mutex> lock(mutex);
       this->ImageCompare(img, prevImg, width, height, 3,
                          diffMax, diffSum, diffAvg);
       this->ImageCompare(prevImg2, prevImg2, width, height, 3,
