@@ -1,5 +1,5 @@
  /*
- * Copyright (C) 2013-2015 Open Source Robotics Foundation
+ * Copyright (C) 2013-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -160,6 +160,21 @@ TEST_F(ModelDatabaseTest, GetModelsThrice)
   EXPECT_EQ(g_onModels2, 3);
 }
 
+/////////////////////////////////////////////////
+TEST_F(ModelDatabaseTest, Version)
+{
+  // add test model cococan to path so that the model database
+  // can pick it up
+  gazebo::common::SystemPaths::Instance()->AddModelPaths(
+    CMAKE_SOURCE_DIR "/test/models/testdb");
+
+  std::string uri = "model://cococan";
+  std::string model;
+  model = gazebo::common::ModelDatabase::Instance()->GetModelFile(uri);
+  // this model hias multiple sdf files. 1_4 is the correct one
+  EXPECT_TRUE(model.find("model-1_4.sdf") != std::string::npos);
+}
+
 /////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
