@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,10 @@
 #include "gazebo/test/ServerFixture.hh"
 #include "gazebo/test/helper_physics_generator.hh"
 
+#ifdef HAVE_BULLET
+#include "gazebo/physics/bullet/bullet_math_inc.h"
+#endif
+
 #define PHYSICS_TOL 1e-2
 using namespace gazebo;
 
@@ -34,6 +38,7 @@ class PhysicsMsgsTest : public ServerFixture,
   public: void LinkPose(const std::string &_physicsEngine);
   public: void SimpleShapeResize(const std::string &_physicsEngine);
   public: void LinkVisualMsg(const std::string &_physicsEngine);
+  public: void JointMsg(const std::string &_physicsEngine);
 };
 
 /////////////////////////////////////////////////
@@ -100,7 +105,7 @@ void PhysicsMsgsTest::MoveTool(const std::string &_physicsEngine)
   physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
-  physics->SetGravity(math::Vector3::Zero);
+  world->SetGravity(ignition::math::Vector3d::Zero);
 
   // spawn a box
   std::string name = "test_box";
@@ -314,7 +319,7 @@ void PhysicsMsgsTest::LinkPose(const std::string &_physicsEngine)
   physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
-  physics->SetGravity(math::Vector3::Zero);
+  world->SetGravity(ignition::math::Vector3d::Zero);
 
   // advertise on "~/model/modify"
   transport::PublisherPtr modelPub =
@@ -497,7 +502,7 @@ void PhysicsMsgsTest::SimpleShapeResize(const std::string &_physicsEngine)
     else
     {
       // Use physics API to resize
-      model->SetScale(scaleFactor * math::Vector3::One);
+      model->SetScale(scaleFactor * ignition::math::Vector3d::One);
     }
   }
 
@@ -533,8 +538,20 @@ void PhysicsMsgsTest::SimpleShapeResize(const std::string &_physicsEngine)
       pose1 = model->GetWorldPose();
       x0 = modelPos[name].x;
       y0 = modelPos[name].y;
-      EXPECT_NEAR(pose1.pos.x, x0, PHYSICS_TOL);
-      EXPECT_NEAR(pose1.pos.y, y0, PHYSICS_TOL);
+      double xTolerance = PHYSICS_TOL;
+      double yTolerance = PHYSICS_TOL;
+#ifdef HAVE_BULLET
+      if (_physicsEngine == "bullet" && sizeof(btScalar) == 4)
+      {
+        if (name.find("test_box") != std::string::npos)
+        {
+          xTolerance *= 1.6;
+          yTolerance *= 2.3;
+        }
+      }
+#endif
+      EXPECT_NEAR(pose1.pos.x, x0, xTolerance);
+      EXPECT_NEAR(pose1.pos.y, y0, yTolerance);
       EXPECT_NEAR(pose1.pos.z, 0.5*scaleFactor, PHYSICS_TOL);
     }
     else
@@ -677,6 +694,7 @@ void PhysicsMsgsTest::SimpleShapeResize(const std::string &_physicsEngine)
     }
   }
 }
+
 ////////////////////////////////////////////////////////////////////////
 void PhysicsMsgsTest::LinkVisualMsg(const std::string &_physicsEngine)
 {
@@ -761,6 +779,227 @@ void PhysicsMsgsTest::LinkVisualMsg(const std::string &_physicsEngine)
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+void PhysicsMsgsTest::JointMsg(const std::string &_physicsEngine)
+{
+  // load an empty world
+  Load("worlds/empty.world", true, _physicsEngine);
+  physics::WorldPtr world = physics::get_world("default");
+  ASSERT_TRUE(world != NULL);
+
+  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  ASSERT_TRUE(physics != NULL);
+  EXPECT_EQ(physics->GetType(), _physicsEngine);
+
+  std::ostringstream sdfStream;
+  sdfStream << "<sdf version='" << SDF_VERSION << "'>"
+    << "<model name ='joint_msg_test'>"
+    << "<link name ='link1'>"
+    << "  <pose>0 0 0 0 0 0</pose>"
+    << "  <collision name ='geom'>"
+    << "    <geometry>"
+    << "      <box><size>1 1 1</size></box>"
+    << "    </geometry>"
+    << "  </collision>"
+    << "</link>"
+    << "<link name ='link2'>"
+    << "  <pose>0 1 0 0 0 0</pose>"
+    << "  <collision name ='geom'>"
+    << "    <geometry>"
+    << "      <box><size>1 1 1</size></box>"
+    << "    </geometry>"
+    << "  </collision>"
+    << "</link>"
+    << "<link name ='link3'>"
+    << "  <pose>0 0 1 0 0 0</pose>"
+    << "  <collision name ='geom'>"
+    << "    <geometry>"
+    << "      <box><size>1 1 1</size></box>"
+    << "    </geometry>"
+    << "  </collision>"
+    << "</link>"
+    << "<joint name='revolute_joint' type='revolute'>"
+    << "  <parent>link1</parent>"
+    << "  <child>link2</child>"
+    << "  <pose>0 0 0 0 0 0</pose>"
+    << "  <axis>"
+    << "    <xyz>1 0 0</xyz>"
+    << "    <use_parent_model_frame>0</use_parent_model_frame>"
+    << "    <limit>"
+    << "      <lower>-1</lower>"
+    << "      <upper>1</upper>"
+    << "      <effort>-1</effort>"
+    << "      <velocity>-1</velocity>"
+    << "    </limit>"
+    << "    <dynamics>"
+    << "      <damping>0.2</damping>"
+    << "      <friction>0.1</friction>"
+    << "    </dynamics>"
+    << "  </axis>"
+    << "</joint>"
+    << "<joint name='screw_joint' type='screw'>"
+    << "  <parent>link2</parent>"
+    << "  <child>link3</child>"
+    << "  <pose>0 0.2 0 0 0 0</pose>"
+    << "  <axis>"
+    << "    <xyz>0 1 0</xyz>"
+    << "    <use_parent_model_frame>0</use_parent_model_frame>"
+    << "    <limit>"
+    << "      <lower>-2</lower>"
+    << "      <upper>2</upper>"
+    << "      <effort>-0.7</effort>"
+    << "      <velocity>-1</velocity>"
+    << "    </limit>"
+    << "    <dynamics>"
+    << "      <damping>0.3</damping>"
+    << "      <friction>0.2</friction>"
+    << "    </dynamics>"
+    << "  </axis>"
+    << "  <thread_pitch>2</thread_pitch>"
+    << "</joint>"
+    << "<joint name='gearbox_joint' type='gearbox'>"
+    << "  <parent>link3</parent>"
+    << "  <child>link1</child>"
+    << "  <pose>0 0 0.1 0 0 0</pose>"
+    << "  <axis>"
+    << "    <xyz>1 0 0</xyz>"
+    << "    <use_parent_model_frame>0</use_parent_model_frame>"
+    << "    <limit>"
+    << "      <lower>-1e6</lower>"
+    << "      <upper>1e6</upper>"
+    << "      <effort>-0.9</effort>"
+    << "      <velocity>-0.1</velocity>"
+    << "    </limit>"
+    << "    <dynamics>"
+    << "      <damping>0.4</damping>"
+    << "      <friction>0.11</friction>"
+    << "    </dynamics>"
+    << "  </axis>"
+    << "  <axis2>"
+    << "    <xyz>0 0 1</xyz>"
+    << "    <use_parent_model_frame>1</use_parent_model_frame>"
+    << "    <limit>"
+    << "      <lower>-1e3</lower>"
+    << "      <upper>1e3</upper>"
+    << "      <effort>-0.8</effort>"
+    << "      <velocity>-0.2</velocity>"
+    << "    </limit>"
+    << "    <dynamics>"
+    << "      <damping>0.23</damping>"
+    << "      <friction>0.32</friction>"
+    << "    </dynamics>"
+    << "  </axis2>"
+    << "  <gearbox_ratio>6.6</gearbox_ratio>"
+    << "  <gearbox_reference_body>link_2</gearbox_reference_body>"
+    << "</joint>"
+    << "</model>"
+    << "</sdf>";
+  SpawnSDF(sdfStream.str());
+
+  physics::ModelPtr model;
+  model = world->GetModel("joint_msg_test");
+  EXPECT_TRUE(model != NULL);
+  msgs::Model msg;
+  model->FillMsg(msg);
+
+  // only ode supports gearbox joint
+  int jointSize = 2;
+  if (_physicsEngine == "ode")
+    jointSize = 3;
+
+  EXPECT_EQ(msg.joint_size(), jointSize);
+
+  {
+    msgs::Joint jointMsg = msg.joint(0);
+    EXPECT_EQ(jointMsg.name(), "joint_msg_test::revolute_joint");
+    EXPECT_EQ(jointMsg.parent(), "joint_msg_test::link1");
+    EXPECT_EQ(jointMsg.child(), "joint_msg_test::link2");
+    EXPECT_EQ(msgs::ConvertIgn(jointMsg.pose()),
+        ignition::math::Pose3d(0, 0, 0, 0, 0, 0));
+    EXPECT_TRUE(jointMsg.has_axis1());
+    EXPECT_TRUE(!jointMsg.has_axis2());
+    msgs::Axis axis1Msg = jointMsg.axis1();
+    EXPECT_EQ(msgs::ConvertIgn(axis1Msg.xyz()),
+        ignition::math::Vector3d(1, 0, 0));
+    EXPECT_EQ(axis1Msg.use_parent_model_frame(), false);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_lower(), -1);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_upper(), 1);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_effort(), -1);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_velocity(), -1);
+    EXPECT_DOUBLE_EQ(axis1Msg.damping(), 0.2);
+    // only ode and bullet return correct hinge friction param value
+    if (_physicsEngine == "ode" || _physicsEngine == "bullet")
+      EXPECT_FLOAT_EQ(axis1Msg.friction(), 0.1);
+  }
+
+  {
+    msgs::Joint jointMsg = msg.joint(1);
+    EXPECT_EQ(jointMsg.name(), "joint_msg_test::screw_joint");
+    EXPECT_EQ(jointMsg.parent(), "joint_msg_test::link2");
+    EXPECT_EQ(jointMsg.child(), "joint_msg_test::link3");
+    EXPECT_EQ(msgs::ConvertIgn(jointMsg.pose()),
+        ignition::math::Pose3d(0, 0.2, 0, 0, 0, 0));
+    EXPECT_TRUE(jointMsg.has_axis1());
+    EXPECT_TRUE(jointMsg.has_axis2());
+    msgs::Axis axis1Msg = jointMsg.axis1();
+    EXPECT_EQ(msgs::ConvertIgn(axis1Msg.xyz()),
+        ignition::math::Vector3d(0, 1, 0));
+    EXPECT_EQ(axis1Msg.use_parent_model_frame(), false);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_lower(), -2);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_upper(), 2);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_effort(), -0.7);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_velocity(), -1);
+    EXPECT_DOUBLE_EQ(axis1Msg.damping(), 0.3);
+    // only ode returns correct screw friction param value
+    if (_physicsEngine == "ode")
+      EXPECT_DOUBLE_EQ(axis1Msg.friction(), 0.2);
+    msgs::Joint::Screw screwMsg = jointMsg.screw();
+    EXPECT_DOUBLE_EQ(screwMsg.thread_pitch(), 2);
+  }
+
+  // only ode supports gearbox joints
+  if (_physicsEngine == "bullet" || _physicsEngine == "simbody"
+      || _physicsEngine == "dart")
+    return;
+
+  {
+    msgs::Joint jointMsg = msg.joint(2);
+    EXPECT_EQ(jointMsg.name(), "joint_msg_test::gearbox_joint");
+    EXPECT_EQ(jointMsg.parent(), "joint_msg_test::link3");
+    EXPECT_EQ(jointMsg.child(), "joint_msg_test::link1");
+    EXPECT_EQ(msgs::ConvertIgn(jointMsg.pose()),
+        ignition::math::Pose3d(0, 0, 0.1, 0, 0, 0));
+    EXPECT_TRUE(jointMsg.has_axis1());
+    EXPECT_TRUE(jointMsg.has_axis2());
+    msgs::Axis axis1Msg = jointMsg.axis1();
+    EXPECT_EQ(msgs::ConvertIgn(axis1Msg.xyz()),
+        ignition::math::Vector3d(1, 0, 0));
+    EXPECT_EQ(axis1Msg.use_parent_model_frame(), false);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_lower(), -1e6);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_upper(), 1e6);
+    EXPECT_DOUBLE_EQ(axis1Msg.limit_effort(), -0.9);
+    EXPECT_EQ(axis1Msg.limit_velocity(), -0.1);
+    EXPECT_DOUBLE_EQ(axis1Msg.damping(), 0.4);
+    // gearbox friction param does not return correct value
+    // EXPECT_DOUBLE_EQ(axis1Msg.friction(), 0.11);
+    msgs::Axis axis2Msg = jointMsg.axis2();
+    EXPECT_EQ(msgs::ConvertIgn(axis2Msg.xyz()),
+        ignition::math::Vector3d(0, 0, 1));
+    EXPECT_EQ(axis2Msg.use_parent_model_frame(), true);
+    EXPECT_DOUBLE_EQ(axis2Msg.limit_lower(), -1e3);
+    EXPECT_DOUBLE_EQ(axis2Msg.limit_upper(), 1e3);
+    EXPECT_DOUBLE_EQ(axis2Msg.limit_effort(), -0.8);
+    EXPECT_DOUBLE_EQ(axis2Msg.limit_velocity(), -0.2);
+    EXPECT_DOUBLE_EQ(axis2Msg.damping(), 0.23);
+    // gearbox friction param does not return correct value
+    // EXPECT_DOUBLE_EQ(axis2Msg.friction(), 0.32);
+    msgs::Joint::Gearbox gearboxMsg = jointMsg.gearbox();
+    EXPECT_EQ(gearboxMsg.gearbox_reference_body(),
+         "link_2");
+    EXPECT_DOUBLE_EQ(gearboxMsg.gearbox_ratio(), 6.6);
+  }
+}
+
 /////////////////////////////////////////////////
 TEST_P(PhysicsMsgsTest, SetGravity)
 {
@@ -797,6 +1036,12 @@ TEST_P(PhysicsMsgsTest, LinkVisualMsg)
   LinkVisualMsg(GetParam());
 }
 
+/////////////////////////////////////////////////
+TEST_P(PhysicsMsgsTest, JointMsg)
+{
+  JointMsg(GetParam());
+}
+
 INSTANTIATE_TEST_CASE_P(PhysicsEngines, PhysicsMsgsTest,
                         PHYSICS_ENGINE_VALUES);
 
