@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -54,7 +54,9 @@ void ReceiveSimEvent(ConstSimEventPtr &_msg)
   g_event_type = _msg->type();
   g_event_name = _msg->name();
   g_event_data = _msg->data();
-  gzdbg << "ReceiveSimEvent " << g_event_type << std::endl;
+  gzdbg << "ReceiveSimEvent " << g_event_type
+        << " " << g_event_name
+        << std::endl;
 }
 
 // get the count in a thread safe way
@@ -81,8 +83,8 @@ std::string GetEventData()
 // waits for one or multiple events. if the expected number is
 // specified, then the function can return early
 unsigned int WaitForNewEvent(unsigned int current,
-                             unsigned int max_tries = 50,
-                             unsigned int ms = 10)
+                             unsigned int max_tries = 20,
+                             unsigned int ms = 100)
 {
   for (unsigned int i = 0; i < max_tries; i++)
   {
@@ -254,9 +256,10 @@ void SimEventsTest::JointEventSource(const std::string &_physicsEngine)
 
   // check that after position, we have received a new event
   unsigned int count_before = GetEventCount();
-  // rotate joint
 
-  joint->SetPosition(0, IGN_PI);
+  // rotate joint
+  joint->SetPosition(0, 3.1);
+
   // check for event
   unsigned int count_after = WaitForNewEvent(count_before);
   EXPECT_GT(count_after, count_before);
