@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2015 Open Source Robotics Foundation
+ * Copyright (C) 2013-2016 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -97,7 +97,7 @@ void init()
   {
     boost::filesystem::path worldFilePath = TEST_PATH;
     worldFilePath = worldFilePath / "worlds" / "simple_arm_test.world";
-    if (execlp("gzserver", worldFilePath.string().c_str(),
+    if (execlp("gzserver", "gzserver", worldFilePath.string().c_str(),
         "--iters", "60000", NULL) < 0)
     {
       gzerr << "Failed to start the gazebo server.\n";
@@ -281,6 +281,21 @@ TEST_F(gzTest, Model)
   // Run the transport loop: starts a new thread
   gazebo::transport::run();
 
+  // Test model info on existing model
+  {
+    std::string modelInfo = custom_exec_str("gz model -m simple_arm -i");
+
+    // Check that a few values exist. We don't check the sha1 value
+    // because a few values, such as pose, are dynamic.
+    EXPECT_TRUE(modelInfo.find("name: \"simple_arm\"") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find(
+        "name: \"simple_arm::arm_wrist_roll::arm_wrist_roll_geom_visual\"")
+        != std::string::npos);
+    EXPECT_TRUE(modelInfo.find(
+        "name: \"simple_arm::arm_wrist_lift::arm_wrist_lift_geom\"")
+        != std::string::npos);
+  }
+
   // Test model move
   {
     waitForMsg("gz model -w default -m simple_arm "
@@ -297,8 +312,19 @@ TEST_F(gzTest, Model)
   {
     std::string filename = std::string(TEST_PATH) + "/models/box.sdf";
 
-    waitForMsg("gz model -w default -m my_box -f " + filename);
+    g_msgDebugOut.clear();
+    custom_exec_str("gz model -w default -m my_box -f " + filename);
 
+    int maxSleep = 10;
+    int sleep = 0;
+    while (g_msgDebugOut.empty() && sleep < maxSleep)
+    {
+      gazebo::common::Time::MSleep(100);
+      sleep++;
+    }
+    EXPECT_FALSE(g_msgDebugOut.empty());
+
+    // Create similar message to compare
     sdf::SDFPtr sdf(new sdf::SDF());
     EXPECT_TRUE(sdf::init(sdf));
 
@@ -314,14 +340,55 @@ TEST_F(gzTest, Model)
     EXPECT_EQ(g_msgDebugOut, msg.DebugString());
   }
 
+  // Test model info of inserted model to verify insertion
+  {
+    std::string modelInfo = custom_exec_str("gz model -m my_box -i");
+
+    // Check that a few values exist. We don't check the sha1 value
+    // because a few values, such as pose, are dynamic.
+    EXPECT_TRUE(modelInfo.find("name: \"my_box\"") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("id: 56") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("is_static: false") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("name: \"my_box::link::collision\"")
+        != std::string::npos);
+  }
+
+  // Test model delete
+  {
+    waitForMsg("gz model -w default -m my_box -d");
+
+    EXPECT_NE(g_msgDebugOut.find("entity_delete"), std::string::npos);
+    EXPECT_NE(g_msgDebugOut.find("my_box"), std::string::npos);
+  }
+
+  // Test model info of deleted model to verify deletion
+  {
+    // Make sure the error message is output.
+    std::string modelInfo = custom_exec_str("gz model -m my_box -i");
+    EXPECT_EQ(modelInfo,
+        "Unable to get info on model[my_box] in the world[default]\n");
+  }
+
   // Test model spawn from string
   {
     std::string filename = std::string(TEST_PATH) + "/models/box.sdf";
 
     std::string cmd = "cat ";
     cmd += filename + " | gz model -w default -m my_box -s";
-    waitForMsg(cmd);
 
+    g_msgDebugOut.clear();
+    custom_exec_str(cmd);
+
+    int maxSleep = 10;
+    int sleep = 0;
+    while (g_msgDebugOut.empty() && sleep < maxSleep)
+    {
+      gazebo::common::Time::MSleep(100);
+      sleep++;
+    }
+    EXPECT_FALSE(g_msgDebugOut.empty());
+
+    // Create similar message to compare
     sdf::SDFPtr sdf(new sdf::SDF());
     EXPECT_TRUE(sdf::init(sdf));
 
@@ -337,6 +404,14 @@ TEST_F(gzTest, Model)
     EXPECT_EQ(g_msgDebugOut, msg.DebugString());
   }
 
+  // Check there were no extra copies of the model inserted
+  {
+    // Make sure the error message is output.
+    std::string modelInfo = custom_exec_str("gz model -m my_box_0 -i");
+    EXPECT_EQ(modelInfo,
+        "Unable to get info on model[my_box_0] in the world[default]\n");
+  }
+
   // Test model info and pose
   {
     // Make sure the error message is output.
@@ -350,7 +425,8 @@ TEST_F(gzTest, Model)
     // Check that a few values exist. We don't check the sha1 value
     // because a few values, such as pose, are dynamic.
     EXPECT_TRUE(modelInfo.find("name: \"my_box\"") != std::string::npos);
-    EXPECT_TRUE(modelInfo.find("id: 9") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("id: 62") != std::string::npos);
+    EXPECT_TRUE(modelInfo.find("is_static: false") != std::string::npos);
     EXPECT_TRUE(modelInfo.find("name: \"my_box::link::collision\"")
         != std::string::npos);
 
@@ -658,26 +734,29 @@ TEST_F(gzTest, SDF)
   path /= "worlds/box_plane_low_friction_test.world";
 
   {
-    // Check empty.world
+    // Check box_plane_low_friction_test.world
     std::string output =
       custom_exec_str(std::string("gz sdf -k ") + path.string());
     EXPECT_EQ(output, "Check complete\n");
   }
 
-  {
-    // Print empty.world
-    // Regenerate using:
-    // gz sdf -p test/worlds/empty_different_name.world
-    // | sed ':a;N;$!ba;s/\n/\\n/g' | sed 's/"/\\"/g'
-    std::string output =
-      custom_exec_str(std::string("gz sdf -p ") + path.string());
-    std::string shasum = gazebo::common::get_sha1<std::string>(output);
-    EXPECT_EQ(shasum, "ea127f9858a5e07c40ef6d949ef6113236adddfa");
-  }
+  // The checksums are changing too often for this test to be useful
+  // {
+  //   // Print box_plane_low_friction_test.world
+  //   // Regenerate using:
+  //   // gz sdf -p test/worlds/box_plane_low_friction_test.world
+  //   // | shasum
+  //   std::string output =
+  //     custom_exec_str(std::string("gz sdf -p ") + path.string());
+  //   std::string shasum = gazebo::common::get_sha1<std::string>(output);
+  //   EXPECT_EQ(shasum, "81960fabd921e1832aeb15d8b1ac39754c0eed81");
+  // }
 
   path = PROJECT_BINARY_PATH;
   path = path / "test" / "sdf_convert_test.world";
   std::ofstream file(path.string().c_str(), std::ios::out);
+  EXPECT_TRUE(file.is_open());
+
   file << "<?xml version='1.0' ?>"
     "<sdf version='1.3'>"
     "<world name='default'>"
